import glob

json_files = glob.glob("tests/**/output/**/*.json", recursive=True)
html_files = glob.glob("tests/**/output/**/*.html", recursive=True)

html_list = ""
for f_ in html_files:
    html_list += '\t<li><a href="{}">{}</li>\n'.format(f_[6:], f_.split(".")[-2],)

json_list = ""
for f_ in json_files:
    json_list += '\t<li><a href="{}">{}</li>\n'.format(f_[6:], f_.split(".")[-2],)

html_file = """
<html>
<body>
  <h3>HTML</h3>
  <ul>
    {}
  </ul>
  <br/><br/>
  <h3>JSON</h3>
  <ul>
    {}
  </ul>
</body>
</html>
""".format(
    html_list, json_list
)

print(html_file)



import json
from collections import OrderedDict

import pytest

import great_expectations as ge
import great_expectations.render as render
from great_expectations.core import ExpectationSuiteValidationResult
from great_expectations.data_context.util import file_relative_path
from great_expectations.render.renderer import ProfilingResultsPageRenderer
from great_expectations.render.types import (
    RenderedGraphContent,
    RenderedHeaderContent,
    RenderedSectionContent,
    RenderedStringTemplateContent,
    RenderedTableContent,
    TextContent,
    ValueListContent,
)
from great_expectations.render.view import DefaultJinjaPageView


@pytest.fixture()
def validation_results():
    with open("./tests/test_sets/expected_cli_results_default.json", "r") as infile:
        return json.load(infile, object_pairs_hook=OrderedDict)


@pytest.fixture()
def expectations():
    with open("./tests/test_sets/titanic_expectations.json", "r") as infile:
        return json.load(infile, object_pairs_hook=OrderedDict)


# noinspection PyPep8Naming
@pytest.mark.smoketest
@pytest.mark.rendered_output
def test_render_DefaultJinjaPageView_meta_info():
    validation_results = ExpectationSuiteValidationResult(
        **{
            "results": [],
            "statistics": {
                "evaluated_expectations": 156,
                "successful_expectations": 139,
                "unsuccessful_expectations": 17,
                "success_percent": 89.1025641025641,
            },
            "meta": {
                "great_expectations_version": "0.7.0-beta",
                "data_asset_name": "datasource/generator/tetanusvaricella",
                "expectation_suite_name": "my_suite",
                "run_id": "2019-06-25T14:58:09.960521",
                "batch_kwargs": {
                    "path": "/Users/user/project_data/public_healthcare_datasets/tetanusvaricella/tetvardata.csv",
                    "timestamp": 1561474688.693565,
                },
            },
        }
    )

    document = ProfilingResultsPageRenderer().render(validation_results)
    html = DefaultJinjaPageView().render(document)
    with open(
        file_relative_path(
            __file__, "./output/test_render_DefaultJinjaPageView_meta_info.html)"
        ),
        "w",
    ) as outfile:
        outfile.write(html)


def test_render_section_page():
    section = RenderedSectionContent(
        **{
            "section_name": None,
            "content_blocks": [
                RenderedHeaderContent(
                    **{"content_block_type": "header", "header": "Overview",}
                ),
                RenderedTableContent(
                    **{
                        "content_block_type": "table",
                        "header": "Dataset info",
                        "table": [
                            ["Number of variables", "12"],
                            ["Number of observations", "891"],
                        ],
                        "styling": {
                            "classes": ["col-6", "table-responsive"],
                            "styles": {"margin-top": "20px"},
                            "body": {"classes": ["table", "table-sm"]},
                        },
                    }
                ),
            ],
        }
    ).to_json_dict()

    rendered_doc = ge.render.view.view.DefaultJinjaSectionView().render(
        {"section": section, "section_loop": {"index": 1},}
    )  # .replace(" ", "").replace("\t", "").replace("\n", "")

    print(rendered_doc)

    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """<div id="section-1" class="ge-section container-fluid mb-1 pb-1 pl-sm-3 px-0">
    <div class="row" >
<div id="content-block-1" >
    <div id="content-block-1-header" >
        <h5>
            Overview
        </h5>
    </div>
</div>
<div id="content-block-2" class="col-6 table-responsive" style="margin-top:20px;" >
    <div id="content-block-2-header" >
            <h5>
                Dataset info
            </h5>
        </div>
<table
  id="content-block-2-body"
  class="table table-sm"
  data-toggle="table"
>
      <thead hidden>
        <tr>
            <th>
            </th>
            <th>
            </th>
        </tr>
      </thead>
    <tbody>
      <tr>
          <td id="content-block-2-cell-1-1" ><div class="show-scrollbars">Number of variables</div></td><td id="content-block-2-cell-1-2" ><div class="show-scrollbars">12</div></td></tr><tr>
          <td id="content-block-2-cell-2-1" ><div class="show-scrollbars">Number of observations</div></td><td id="content-block-2-cell-2-2" ><div class="show-scrollbars">891</div></td></tr></tbody>
</table>
</div>
    </div>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


def test_rendering_components_without_section_loop_index():
    header_component_content = RenderedHeaderContent(
        **{
            # "component_type": "header",
            "content_block_type": "header",
            "header": "Overview",
        }
    ).to_json_dict()
    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {"content_block": header_component_content, "content_block_loop": {"index": 2},}
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="content-block-2" >
    <div id="content-block-2-header" >
        <h5>
            Overview
        </h5>
    </div>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )

    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {"content_block": header_component_content,}
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="content-block" >
    <div id="content-block-header" >
        <h5>
            Overview
        </h5>
    </div>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )

    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {"content_block": header_component_content, "section_loop": {"index": 3},}
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="content-block" >
    <div id="content-block-header" >
        <h5>
            Overview
        </h5>
    </div>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


def test_rendering_components_with_styling():
    # Medium-complicated example to verify that all the things are correctly piped to all the places

    header_component_content = RenderedTableContent(
        **{
            "content_block_type": "table",
            "header": RenderedStringTemplateContent(
                **{
                    "content_block_type": "string_template",
                    "string_template": {
                        "template": "$var1 $var2 $var3",
                        "params": {"var1": "AAA", "var2": "BBB", "var3": "CCC",},
                        "styling": {
                            "default": {"classes": ["x"]},
                            "params": {"var1": {"classes": ["y"]}},
                        },
                    },
                }
            ),
            "subheader": RenderedStringTemplateContent(
                **{
                    "content_block_type": "string_template",
                    "string_template": {
                        "template": "$var1 $var2 $var3",
                        "params": {"var1": "aaa", "var2": "bbb", "var3": "ccc",},
                        "styling": {
                            "default": {"classes": ["xx"]},
                            "params": {"var1": {"classes": ["yy"]}},
                        },
                    },
                }
            ),
            "table": [["Mean", "446"], ["Minimum", "1"],],
            "styling": {
                "classes": ["root_foo"],
                "styles": {"root": "bar"},
                "attributes": {"root": "baz"},
                "header": {
                    "classes": ["header_foo"],
                    "styles": {"header": "bar"},
                    "attributes": {"header": "baz"},
                },
                "subheader": {
                    "classes": ["subheader_foo"],
                    "styles": {"subheader": "bar"},
                    "attributes": {"subheader": "baz"},
                },
                "body": {
                    "classes": ["body_foo"],
                    "styles": {"body": "bar"},
                    "attributes": {"body": "baz"},
                },
            },
        }
    ).to_json_dict()
    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": header_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")

    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" class="root_foo" root="baz" style="root:bar;" >
    <div id="section-1-content-block-2-header" class="header_foo" header="baz" style="header:bar;" >
            <div>
                <span >
                    <span class="y" >AAA</span> <span class="x" >BBB</span> <span class="x" >CCC</span>
                </span>
            </div>
            <div id="section-1-content-block-2-subheader" class="subheader_foo" subheader="baz" style="subheader:bar;" >

                <span >
                    <span class="yy" >aaa</span> <span class="xx" >bbb</span> <span class="xx" >ccc</span>
                </span>
            </div>
        </div>
<table
  id="section-1-content-block-2-body"
  class="body_foo" body="baz" style="body:bar;"
  data-toggle="table"
>
      <thead hidden>
        <tr>
            <th>
            </th>
            <th>
            </th>
        </tr>
      </thead>
    <tbody>
      <tr>
          <td id="section-1-content-block-2-cell-1-1" ><div class="show-scrollbars">Mean</div></td><td id="section-1-content-block-2-cell-1-2" ><div class="show-scrollbars">446</div></td></tr><tr>
          <td id="section-1-content-block-2-cell-2-1" ><div class="show-scrollbars">Minimum</div></td><td id="section-1-content-block-2-cell-2-2" ><div class="show-scrollbars">1</div></td></tr></tbody>
</table>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


# Test all the component types ###
def test_render_header_component():
    header_component_content = RenderedHeaderContent(
        **{
            # "component_type": "header",
            "content_block_type": "header",
            "header": "Overview",
        }
    ).to_json_dict()
    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": header_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" >
    <div id="section-1-content-block-2-header" >
        <h5>
            Overview
        </h5>
    </div>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


def test_render_table_component():
    table_component_content = RenderedTableContent(
        **{
            "content_block_type": "table",
            "header": "Overview",
            "table": [["Mean", "446"], ["Minimum", "1"],],
            "styling": {"classes": ["col-4"],},
        }
    ).to_json_dict()
    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": table_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" class="col-4" >
    <div id="section-1-content-block-2-header" >
        <h5>
            Overview
        </h5>
    </div>
<table
  id="section-1-content-block-2-body"
  data-toggle="table"
>
      <thead hidden>
        <tr>
            <th>
            </th>
            <th>
            </th>
        </tr>
      </thead>
    <tbody>
      <tr>
          <td id="section-1-content-block-2-cell-1-1" ><div class="show-scrollbars">Mean</div></td><td id="section-1-content-block-2-cell-1-2" ><div class="show-scrollbars">446</div></td></tr><tr>
          <td id="section-1-content-block-2-cell-2-1" ><div class="show-scrollbars">Minimum</div></td><td id="section-1-content-block-2-cell-2-2" ><div class="show-scrollbars">1</div></td></tr></tbody>
</table>

</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


def test_render_value_list():
    value_list_component_content = ValueListContent(
        **{
            "content_block_type": "value_list",
            "header": "Example values",
            "value_list": [
                {
                    "content_block_type": "string_template",
                    "string_template": {
                        "template": "$value",
                        "params": {"value": "0"},
                        "styling": {"default": {"classes": ["badge", "badge-info"]}},
                    },
                },
                {
                    "content_block_type": "string_template",
                    "string_template": {
                        "template": "$value",
                        "params": {"value": "1"},
                        "styling": {"default": {"classes": ["badge", "badge-info"]}},
                    },
                },
            ],
            "styling": {"classes": ["col-4"], "styles": {"margin-top": "20px"}},
        }
    ).to_json_dict()

    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": value_list_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" class="col-4" style="margin-top:20px;" >
    <div id="section-1-content-block-2-header" >
            <h5>
                Example values
            </h5>
        </div>
<p id="section-1-content-block-2-body" >
                <span >
                    <span class="badge badge-info" >0</span>
                </span>
                <span >
                    <span class="badge badge-info" >1</span>
                </span>
    </p>
</div>""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


def test_render_graph():
    graph_component_content = RenderedGraphContent(
        **{
            "content_block_type": "graph",
            "header": "Histogram",
            "graph": '{"$schema": "https://vega.github.io/schema/vega-lite/v2.6.0.json", "autosize": "fit", "config": {"view": {"height": 300, "width": 400}}, "data": {"name": "data-a681d02fb484e64eadd9721b37015d5b"}, "datasets": {"data-a681d02fb484e64eadd9721b37015d5b": [{"bins": 3.7, "weights": 5.555555555555555}, {"bins": 10.8, "weights": 3.439153439153439}, {"bins": 17.9, "weights": 17.857142857142858}, {"bins": 25.0, "weights": 24.206349206349206}, {"bins": 32.0, "weights": 16.137566137566136}, {"bins": 39.1, "weights": 12.3015873015873}, {"bins": 46.2, "weights": 9.788359788359788}, {"bins": 53.3, "weights": 5.423280423280423}, {"bins": 60.4, "weights": 3.439153439153439}, {"bins": 67.5, "weights": 1.8518518518518516}]}, "encoding": {"x": {"field": "bins", "type": "ordinal"}, "y": {"field": "weights", "type": "quantitative"}}, "height": 200, "mark": "bar", "width": 200}',
            "styling": {"classes": ["col-4"]},
        }
    ).to_json_dict()

    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": graph_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" class="col-4" >
    <div id="section-1-content-block-2-header" >
            <h5>
                Histogram
            </h5>
        </div>
<div class="show-scrollbars">
  <div id="section-1-content-block-2-graph" ></div>
</div>
<script>
    // Assign the specification to a local variable vlSpec.
    vlSpec = {"$schema": "https://vega.github.io/schema/vega-lite/v2.6.0.json", "autosize": "fit", "config": {"view": {"height": 300, "width": 400}}, "data": {"name": "data-a681d02fb484e64eadd9721b37015d5b"}, "datasets": {"data-a681d02fb484e64eadd9721b37015d5b": [{"bins": 3.7, "weights": 5.555555555555555}, {"bins": 10.8, "weights": 3.439153439153439}, {"bins": 17.9, "weights": 17.857142857142858}, {"bins": 25.0, "weights": 24.206349206349206}, {"bins": 32.0, "weights": 16.137566137566136}, {"bins": 39.1, "weights": 12.3015873015873}, {"bins": 46.2, "weights": 9.788359788359788}, {"bins": 53.3, "weights": 5.423280423280423}, {"bins": 60.4, "weights": 3.439153439153439}, {"bins": 67.5, "weights": 1.8518518518518516}]}, "encoding": {"x": {"field": "bins", "type": "ordinal"}, "y": {"field": "weights", "type": "quantitative"}}, "height": 200, "mark": "bar", "width": 200};
    // Embed the visualization in the container with id `vis`
    vegaEmbed('#section-1-content-block-2-graph', vlSpec, {
        actions: false
    }).then(result=>console.log(result)).catch(console.warn);
</script>
</div>
""".replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )


def test_render_text():
    text_component_content = TextContent(
        **{
            "content_block_type": "text",
            "header": "Histogram",
            "text": ["hello"],
            "styling": {"classes": ["col-4"]},
        }
    ).to_json_dict()

    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": text_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" class="col-4" >
<div id="section-1-content-block-2-body" class="col-4" >
  <div id="section-1-content-block-2-header" >
            <h5>
                Histogram
            </h5>
        </div>
        <p >hello</p>
    </div>
</div>
    """.replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )

    text_component_content = TextContent(
        **{
            "content_block_type": "text",
            "header": "Histogram",
            "text": ["hello", "goodbye"],
            "styling": {"classes": ["col-4"]},
        }
    ).to_json_dict()

    rendered_doc = ge.render.view.view.DefaultJinjaComponentView().render(
        {
            "content_block": text_component_content,
            "section_loop": {"index": 1},
            "content_block_loop": {"index": 2},
        }
    )
    print(rendered_doc)
    rendered_doc = rendered_doc.replace(" ", "").replace("\t", "").replace("\n", "")
    assert (
        rendered_doc
        == """
<div id="section-1-content-block-2" class="col-4" >
<div id="section-1-content-block-2-body" class="col-4" >
  <div id="section-1-content-block-2-header" >
            <h5>
                Histogram
            </h5>
        </div>
        <p >hello</p>
        <p >goodbye</p>
    </div>
</div>
    """.replace(
            " ", ""
        )
        .replace("\t", "")
        .replace("\n", "")
    )



import logging

import boto3
import pandas as pd
import pytest
from moto import mock_s3

from great_expectations.datasource.batch_kwargs_generator.s3_batch_kwargs_generator import (
    S3GlobReaderBatchKwargsGenerator,
)
from great_expectations.exceptions import BatchKwargsError


@pytest.fixture(scope="module")
def mock_s3_bucket():
    with mock_s3():
        bucket = "test_bucket"

        conn = boto3.resource("s3", region_name="us-east-1")
        conn.create_bucket(Bucket=bucket)
        client = boto3.client("s3", region_name="us-east-1")

        df = pd.DataFrame({"c1": [1, 2, 3], "c2": ["a", "b", "c"]})
        keys = [
            "data/for/you.csv",
            "data/for/me.csv",
            "data/to/you.csv",
            "other/to/you.csv",
            "other/for/you.csv",
            "other/is/you.csv",
            "delta_files/blarg.parquet",
            "data/is/you.csv",
        ]
        for key in keys:
            client.put_object(
                Bucket=bucket, Body=df.to_csv(index=None).encode("utf-8"), Key=key
            )
        yield bucket


@pytest.fixture
def s3_generator(mock_s3_bucket, basic_sparkdf_datasource):
    # We configure a generator that will fetch from (mocked) my_bucket
    # and will use glob patterns to match returned assets into batches of the same asset
    generator = S3GlobReaderBatchKwargsGenerator(
        "my_generator",
        datasource=basic_sparkdf_datasource,
        bucket=mock_s3_bucket,
        reader_options={"sep": ","},
        assets={
            "data": {
                "prefix": "data/",
                "delimiter": "",
                "partition_regex": r"data/for/(.*)\.csv",
                "regex_filter": r"data/for/.*\.csv",
            },
            "data_dirs": {"prefix": "data/", "directory_assets": True},
            "other": {
                "prefix": "other/",
                "regex_filter": r".*/you\.csv",
                "reader_options": {"sep": "\t"},
            },
            "delta_files": {
                "prefix": "delta_files/",
                "regex_filter": r".*/blarg\.parquet",
                "reader_method": "delta",
            },
            "other_empty_delimiter": {
                "prefix": "other/",
                "delimiter": "",
                "regex_filter": r".*/you\.csv",
                "reader_options": {"sep": "\t"},
                "max_keys": 1,
            },
            "dir": {"prefix": "data/", "directory_assets": True,},
            "dir_misconfigured": {"prefix": "data/", "directory_assets": False,},
        },
    )
    yield generator


def test_s3_generator_basic_operation(s3_generator):
    # S3 Generator sees *only* configured assets
    assets = s3_generator.get_available_data_asset_names()
    assert set(assets["names"]) == {
        ("data", "file"),
        ("data_dirs", "file"),
        ("other", "file"),
        ("delta_files", "file"),
        ("other_empty_delimiter", "file"),
        ("dir_misconfigured", "file"),
        ("dir", "file"),
    }

    # We should observe that glob, prefix, delimiter all work together
    # They can be defined in the generator or overridden by a particular asset
    # Under the hood, we use the S3 ContinuationToken options to lazily fetch data
    batch_kwargs = [
        kwargs for kwargs in s3_generator.get_iterator(data_asset_name="data")
    ]
    assert len(batch_kwargs) == 2
    assert batch_kwargs[0]["reader_options"]["sep"] == ","
    assert batch_kwargs[0]["s3"] in [
        "s3a://test_bucket/data/for/you.csv",
        "s3a://test_bucket/data/for/me.csv",
    ]

    # When a prefix and delimiter do not yield objects, there are no objects returned; raise an error
    with pytest.raises(BatchKwargsError) as err:
        batch_kwargs = [
            kwargs for kwargs in s3_generator.get_iterator(data_asset_name="other")
        ]
        # The error should show the common prefixes
    assert "common_prefixes" in err.value.batch_kwargs


def test_s3_generator_incremental_fetch(s3_generator, caplog):
    caplog.set_level(
        logging.DEBUG,
        logger="great_expectations.datasource.batch_kwargs_generator.s3_batch_kwargs_generator",
    )

    # When max_keys is not set, it defaults to 1000, so all items are returned in the first iterator batch,
    # causing only one fetch (and one log entry referencing the startup of the method)
    caplog.clear()
    batch_kwargs = [
        kwargs for kwargs in s3_generator.get_iterator(data_asset_name="data")
    ]
    assert len(caplog.records) == 2
    assert len(batch_kwargs) == 2

    # When there are more items to return than the S3 API returns, we exhaust our iterator and refetch
    # The result is a new log record for each fetch (plus one for entering the method).
    # Since there are three assets matched by 'other_empty_delimiter' we create four additional log entries with three
    # refetch operations
    caplog.clear()
    batch_kwargs = [
        kwargs
        for kwargs in s3_generator.get_iterator(data_asset_name="other_empty_delimiter")
    ]
    assert len(caplog.records) == 4
    assert len(batch_kwargs) == 3


def test_s3_generator_get_directories(s3_generator):
    # Verify that an asset configured to return directories can do so
    batch_kwargs_list = [
        kwargs for kwargs in s3_generator.get_iterator(data_asset_name="data_dirs")
    ]
    assert 3 == len(batch_kwargs_list)
    paths = set([batch_kwargs["s3"] for batch_kwargs in batch_kwargs_list])
    assert {
        "s3a://test_bucket/data/for/",
        "s3a://test_bucket/data/to/",
        "s3a://test_bucket/data/is/",
    } == paths


def test_s3_generator_limit(s3_generator):
    batch_kwargs_list = [
        kwargs for kwargs in s3_generator.get_iterator(data_asset_name="data", limit=10)
    ]
    assert all(["limit" in batch_kwargs for batch_kwargs in batch_kwargs_list])


def test_s3_generator_reader_method_configuration(s3_generator):
    batch_kwargs_list = [
        kwargs
        for kwargs in s3_generator.get_iterator(data_asset_name="delta_files", limit=10)
    ]
    assert batch_kwargs_list[0]["reader_method"] == "delta"


def test_s3_generator_build_batch_kwargs_no_partition_id(s3_generator):
    batch_kwargs = s3_generator.build_batch_kwargs("data")
    assert batch_kwargs["s3"] in [
        "s3a://test_bucket/data/for/you.csv",
        "s3a://test_bucket/data/for/me.csv",
    ]


def test_s3_generator_build_batch_kwargs_partition_id(s3_generator):
    batch_kwargs = s3_generator.build_batch_kwargs("data", "you")
    assert batch_kwargs["s3"] == "s3a://test_bucket/data/for/you.csv"


def test_s3_generator_directory_asset(s3_generator):
    batch_kwargs = s3_generator.build_batch_kwargs("dir")
    assert batch_kwargs["s3"] in [
        "s3a://test_bucket/data/for/",
        "s3a://test_bucket/data/to/",
    ]


def test_s3_generator_misconfigured_directory_asset(s3_generator):
    with pytest.raises(BatchKwargsError) as exc:
        _ = s3_generator.build_batch_kwargs("dir_misconfigured")
    assert "The asset may not be configured correctly." in str(exc.value)



# TODO: ADD TESTS ONCE GET_BATCH IS INTEGRATED!

import pandas as pd
from freezegun import freeze_time

import great_expectations as ge
from great_expectations.data_context import BaseDataContext
from great_expectations.validation_operators.validation_operators import (
    WarningAndFailureExpectationSuitesValidationOperator,
)

from ..test_utils import modify_locale


@modify_locale
@freeze_time("09/26/2019 13:42:41")
def test_errors_warnings_validation_operator_run_slack_query(
    basic_data_context_config_for_validation_operator,
    tmp_path_factory,
    filesystem_csv_4,
):
    project_path = str(tmp_path_factory.mktemp("great_expectations"))

    # NOTE: This setup is almost identical to test_DefaultDataContextAwareValidationOperator.
    # Consider converting to a single fixture.

    data_context = BaseDataContext(
        basic_data_context_config_for_validation_operator, project_path,
    )

    data_context.add_datasource(
        "my_datasource",
        class_name="PandasDatasource",
        batch_kwargs_generators={
            "subdir_reader": {
                "class_name": "SubdirReaderBatchKwargsGenerator",
                "base_directory": str(filesystem_csv_4),
            }
        },
    )

    data_context.create_expectation_suite(expectation_suite_name="f1.failure")
    df = data_context.get_batch(
        expectation_suite_name="f1.failure",
        batch_kwargs=data_context.build_batch_kwargs(
            "my_datasource", "subdir_reader", "f1"
        ),
    )
    df.expect_column_values_to_be_between(column="x", min_value=1, max_value=9)
    failure_expectations = df.get_expectation_suite(discard_failed_expectations=False)
    data_context.save_expectation_suite(
        failure_expectations, expectation_suite_name="f1.failure"
    )

    data_context.create_expectation_suite(expectation_suite_name="f1.warning")
    df = data_context.get_batch(
        expectation_suite_name="f1.warning",
        batch_kwargs=data_context.build_batch_kwargs(
            "my_datasource", "subdir_reader", "f1"
        ),
    )
    df.expect_column_values_to_be_between(column="x", min_value=1, max_value=9)
    df.expect_column_values_to_not_be_null(column="y")
    warning_expectations = df.get_expectation_suite(discard_failed_expectations=False)
    data_context.save_expectation_suite(
        warning_expectations, expectation_suite_name="f1.warning"
    )

    data_context.save_expectation_suite(
        failure_expectations, expectation_suite_name="f2.failure"
    )
    data_context.save_expectation_suite(
        failure_expectations, expectation_suite_name="f3.failure"
    )
    data_context.save_expectation_suite(
        warning_expectations, expectation_suite_name="f2.warning"
    )
    data_context.save_expectation_suite(
        warning_expectations, expectation_suite_name="f3.warning"
    )

    vo = WarningAndFailureExpectationSuitesValidationOperator(
        data_context=data_context,
        action_list=[],
        name="test",
        slack_webhook="https://hooks.slack.com/services/test/slack/webhook",
    )

    my_df_1 = pd.DataFrame({"x": [1, 2, 3, 4, 5], "y": [1, 2, 3, 4, None]})
    my_ge_df_1 = ge.dataset.PandasDataset(
        my_df_1, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-8226-acde48001122"}
    )

    my_df_2 = pd.DataFrame({"x": [1, 2, 3, 4, 99], "y": [1, 2, 3, 4, 5]})
    my_ge_df_2 = ge.dataset.PandasDataset(
        my_df_2, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-8133-acde48001122"}
    )

    my_df_3 = pd.DataFrame({"x": [1, 2, 3, 4, 5], "y": [1, 2, 3, 4, 5]})
    my_ge_df_3 = ge.dataset.PandasDataset(
        my_df_3, batch_kwargs={"ge_batch_id": "82a8de83-e063-11e9-a53d-acde48001122"}
    )

    return_obj = vo.run(
        assets_to_validate=[my_ge_df_1, my_ge_df_2, my_ge_df_3],
        run_id="test_100",
        base_expectation_suite_name="f1",
    )
    slack_query = vo._build_slack_query(return_obj)
    expected_slack_query = {
        "blocks": [
            {"type": "divider"},
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*FailureVsWarning Validation Operator Completed.*",
                },
            },
            {"type": "divider"},
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": "*Status*: Failed :x:"},
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*Batch Id List:* ['ge_batch_id=82a8de83-e063-11e9-8133-acde48001122', "
                    "'ge_batch_id=82a8de83-e063-11e9-8226-acde48001122', "
                    "'ge_batch_id=82a8de83-e063-11e9-a53d-acde48001122']",
                },
            },
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*Failed Batches:* ['f1.failure-ge_batch_id=82a8de83-e063-11e9-8133-acde48001122']",
                },
            },
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": "*Run Name:* test_100"},
            },
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": "*Run Time:* LOCALEDATE"},
            },
            {"type": "divider"},
            {
                "type": "context",
                "elements": [
                    {
                        "type": "mrkdwn",
                        "text": "Learn about FailureVsWarning Validation Operators at https://docs.greatexpectations.i"
                        "o/en/latest/reference/validation_operators/warning_and_failure_expectation_suites_val"
                        "idation_operator.html",
                    }
                ],
            },
        ]
    }

    # We're okay with system variation in locales (OS X likes 24 hour, but not Travis)
    slack_query["blocks"][7]["text"]["text"] = slack_query["blocks"][7]["text"][
        "text"
    ].replace("09/26/2019 13:42:41", "LOCALEDATE")
    slack_query["blocks"][7]["text"]["text"] = slack_query["blocks"][7]["text"][
        "text"
    ].replace("09/26/2019 01:42:41 PM", "LOCALEDATE")
    expected_slack_query["blocks"][7]["text"]["text"] = expected_slack_query["blocks"][
        7
    ]["text"]["text"].replace("09/26/2019 13:42:41", "LOCALEDATE")
    expected_slack_query["blocks"][7]["text"]["text"] = expected_slack_query["blocks"][
        7
    ]["text"]["text"].replace("09/26/2019 01:42:41 PM", "LOCALEDATE")

    import json

    print(json.dumps(slack_query, indent=2))
    print(json.dumps(expected_slack_query, indent=2))
    assert slack_query == expected_slack_query



from freezegun import freeze_time

from great_expectations.core import ExpectationSuiteValidationResult
from great_expectations.render.renderer import SlackRenderer

from ..test_utils import modify_locale


@modify_locale
@freeze_time("09/24/2019 23:18:36")
def test_SlackRenderer():
    validation_result_suite = ExpectationSuiteValidationResult(
        results=[],
        success=True,
        statistics={
            "evaluated_expectations": 0,
            "successful_expectations": 0,
            "unsuccessful_expectations": 0,
            "success_percent": None,
        },
        meta={
            "great_expectations_version": "v0.8.0__develop",
            "data_asset_name": {
                "datasource": "x",
                "generator": "y",
                "generator_asset": "z",
            },
            "expectation_suite_name": "default",
            "run_id": "2019-09-25T060538.829112Z",
        },
    )

    rendered_output = SlackRenderer().render(validation_result_suite)
    print(rendered_output)

    expected_renderer_output = {
        "blocks": [
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": "*Batch Validation Status*: Success :tada:\n*Expectation suite name*: `default`\n*Run ID*: `2019-09-25T060538.829112Z`\n*Batch ID*: `None`\n*Summary*: *0* of *0* expectations were met",
                },
            },
            {"type": "divider"},
            {
                "type": "context",
                "elements": [
                    {
                        "type": "mrkdwn",
                        "text": "Learn how to review validation results in Data Docs: https://docs.greatexpectations.io/en/latest/tutorials/getting_started/set_up_data_docs.html#_getting_started__set_up_data_docs",
                    }
                ],
            },
        ],
        "text": "default: Success :tada:",
    }

    # We're okay with system variation in locales (OS X likes 24 hour, but not Travis)
    expected_renderer_output["blocks"][0]["text"]["text"] = expected_renderer_output[
        "blocks"
    ][0]["text"]["text"].replace("09/24/2019 11:18:36 PM", "LOCALEDATE")
    expected_renderer_output["blocks"][0]["text"]["text"] = expected_renderer_output[
        "blocks"
    ][0]["text"]["text"].replace("09/24/2019 23:18:36", "LOCALEDATE")
    rendered_output["blocks"][0]["text"]["text"] = rendered_output["blocks"][0]["text"][
        "text"
    ].replace("09/24/2019 11:18:36 PM UTC", "LOCALEDATE")
    rendered_output["blocks"][0]["text"]["text"] = rendered_output["blocks"][0]["text"][
        "text"
    ].replace("09/24/2019 23:18:36 UTC", "LOCALEDATE")

    assert rendered_output == expected_renderer_output



import importlib.util
from unittest import mock

import pandas as pd
import pytest

from great_expectations.dataset.sparkdf_dataset import SparkDFDataset


def test_sparkdfdataset_persist(spark_session):
    df = pd.DataFrame({"a": [1, 2, 3]})
    sdf = spark_session.createDataFrame(df)
    sdf.persist = mock.MagicMock()
    _ = SparkDFDataset(sdf, persist=True)
    sdf.persist.assert_called_once()

    sdf = spark_session.createDataFrame(df)
    sdf.persist = mock.MagicMock()
    _ = SparkDFDataset(sdf, persist=False)
    sdf.persist.assert_not_called()

    sdf = spark_session.createDataFrame(df)
    sdf.persist = mock.MagicMock()
    _ = SparkDFDataset(sdf)
    sdf.persist.assert_called_once()


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
@pytest.fixture
def test_dataframe(spark_session):
    from pyspark.sql.types import IntegerType, StringType, StructField, StructType

    schema = StructType(
        [
            StructField("name", StringType(), True),
            StructField("age", IntegerType(), True),
            StructField(
                "address",
                StructType(
                    [
                        StructField("street", StringType(), True),
                        StructField("city", StringType(), True),
                        StructField("house_number", IntegerType(), True),
                    ]
                ),
                False,
            ),
            StructField("name_duplicate", StringType(), True),
            StructField("non.nested", StringType(), True),
        ]
    )
    rows = [
        ("Alice", 1, ("Street 1", "Alabama", 10), "Alice", "a"),
        ("Bob", 2, ("Street 2", "Brooklyn", 11), "Bob", "b"),
        ("Charlie", 3, ("Street 3", "Alabama", 12), "Charlie", "c"),
    ]

    rdd = spark_session.sparkContext.parallelize(rows)

    df = spark_session.createDataFrame(rdd, schema)
    return SparkDFDataset(df, persist=True)


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_values_to_be_of_type(spark_session, test_dataframe):
    """
    data asset expectation
    """
    from pyspark.sql.utils import AnalysisException

    assert test_dataframe.expect_column_values_to_be_of_type(
        "address.street", "StringType"
    ).success
    assert test_dataframe.expect_column_values_to_be_of_type(
        "`non.nested`", "StringType"
    ).success
    assert test_dataframe.expect_column_values_to_be_of_type(
        "name", "StringType"
    ).success
    with pytest.raises(AnalysisException):
        test_dataframe.expect_column_values_to_be_of_type("non.nested", "StringType")


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_values_to_be_of_type(spark_session, test_dataframe):
    """
    data asset expectation
    """
    from pyspark.sql.utils import AnalysisException

    assert test_dataframe.expect_column_values_to_be_of_type(
        "address.street", "StringType"
    ).success
    assert test_dataframe.expect_column_values_to_be_of_type(
        "`non.nested`", "StringType"
    ).success
    assert test_dataframe.expect_column_values_to_be_of_type(
        "name", "StringType"
    ).success
    with pytest.raises(AnalysisException):
        test_dataframe.expect_column_values_to_be_of_type("non.nested", "StringType")


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_values_to_be_in_type_list(spark_session, test_dataframe):
    """
    data asset expectation
    """
    from pyspark.sql.utils import AnalysisException

    assert test_dataframe.expect_column_values_to_be_in_type_list(
        "address.street", ["StringType", "IntegerType"]
    ).success
    assert test_dataframe.expect_column_values_to_be_in_type_list(
        "`non.nested`", ["StringType", "IntegerType"]
    ).success
    assert test_dataframe.expect_column_values_to_be_in_type_list(
        "name", ["StringType", "IntegerType"]
    ).success
    with pytest.raises(AnalysisException):
        test_dataframe.expect_column_values_to_be_of_type("non.nested", "StringType")


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_pair_values_to_be_equal(spark_session, test_dataframe):
    """
    column_pair_map_expectation
    """
    from pyspark.sql.utils import AnalysisException

    assert test_dataframe.expect_column_pair_values_to_be_equal(
        "name", "name_duplicate"
    ).success
    assert not test_dataframe.expect_column_pair_values_to_be_equal(
        "name", "address.street"
    ).success
    assert not test_dataframe.expect_column_pair_values_to_be_equal(
        "name", "`non.nested`"
    ).success

    # Expectation should fail when no `` surround a non-nested column with dot notation
    with pytest.raises(AnalysisException):
        test_dataframe.expect_column_pair_values_to_be_equal("name", "non.nested")


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_pair_values_A_to_be_greater_than_B(
    spark_session, test_dataframe
):
    """
    column_pair_map_expectation
    """
    assert test_dataframe.expect_column_pair_values_A_to_be_greater_than_B(
        "address.house_number", "age"
    ).success
    assert test_dataframe.expect_column_pair_values_A_to_be_greater_than_B(
        "age", "age", or_equal=True
    ).success


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_multicolumn_values_to_be_unique(spark_session, test_dataframe):
    """
    multicolumn_map_expectation
    """
    from pyspark.sql.utils import AnalysisException

    assert test_dataframe.expect_multicolumn_values_to_be_unique(
        ["name", "age"]
    ).success
    assert test_dataframe.expect_multicolumn_values_to_be_unique(
        ["address.street", "name"]
    ).success
    assert test_dataframe.expect_multicolumn_values_to_be_unique(
        ["address.street", "`non.nested`"]
    ).success

    # Expectation should fail when no `` surround a non-nested column with dot notation
    with pytest.raises(AnalysisException):
        test_dataframe.expect_multicolumn_values_to_be_unique(
            ["address.street", "non.nested"]
        )


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_values_to_be_unique(spark_session, test_dataframe):
    """
    column_map_expectation
    """
    from pyspark.sql.utils import AnalysisException

    assert test_dataframe.expect_column_values_to_be_unique("name").success
    assert not test_dataframe.expect_column_values_to_be_unique("address.city").success
    assert test_dataframe.expect_column_values_to_be_unique("`non.nested`").success

    # Expectation should fail when no `` surround a non-nested column with dot notation
    with pytest.raises(AnalysisException):
        test_dataframe.expect_column_values_to_be_unique("non.nested")


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_value_lengths_to_be_between(spark_session, test_dataframe):
    """
    column_map_expectation
    """
    assert test_dataframe.expect_column_value_lengths_to_be_between(
        "name", 3, 7
    ).success
    assert test_dataframe.expect_column_value_lengths_to_be_between(
        "address.street", 1, 10
    ).success


@pytest.mark.skipif(
    importlib.util.find_spec("pyspark") is None, reason="requires the Spark library"
)
def test_expect_column_value_lengths_to_equal(spark_session, test_dataframe):
    """
    column_map_expectation
    """
    assert test_dataframe.expect_column_value_lengths_to_equal("age", 1).success
    assert test_dataframe.expect_column_value_lengths_to_equal(
        "address.street", 8
    ).success



from urllib.parse import parse_qs

import pytest
from pyparsing import ParseException

from great_expectations.core.urn import ge_urn


def test_ge_validations_urn():
    # We should be able to parse validations urns
    urn = (
        "urn:great_expectations:validations:my_suite:expect_something.observed_value:query=s%20tring&query="
        "string3&query2=string2"
    )
    res = ge_urn.parseString(urn)

    assert res["urn_type"] == "validations"
    assert res["expectation_suite_name"] == "my_suite"
    assert res["metric_name"] == "expect_something.observed_value"
    kwargs_dict = parse_qs(res["metric_kwargs"])
    assert kwargs_dict == {"query": ["s tring", "string3"], "query2": ["string2"]}

    # no kwargs is ok
    urn = "urn:great_expectations:validations:my_suite:expect_something.observed_value"
    res = ge_urn.parseString(urn)

    assert res["urn_type"] == "validations"
    assert res["expectation_suite_name"] == "my_suite"
    assert res["metric_name"] == "expect_something.observed_value"
    assert "metric_kwargs" not in res


def test_ge_metrics_urn():
    urn = "urn:great_expectations:metrics:20200403T1234.324Z:my_suite:expect_something.observed_value:column=mycol"
    res = ge_urn.parseString(urn)

    assert res["urn_type"] == "metrics"
    assert res["run_id"] == "20200403T1234.324Z"
    assert res["expectation_suite_name"] == "my_suite"
    assert res["metric_name"] == "expect_something.observed_value"
    kwargs_dict = parse_qs(res["metric_kwargs"])
    assert kwargs_dict == {"column": ["mycol"]}

    # No kwargs is ok
    urn = "urn:great_expectations:metrics:20200403T1234.324Z:my_suite:expect_something.observed_value"
    res = ge_urn.parseString(urn)

    assert res["urn_type"] == "metrics"
    assert res["run_id"] == "20200403T1234.324Z"
    assert res["expectation_suite_name"] == "my_suite"
    assert res["metric_name"] == "expect_something.observed_value"
    assert "kwargs_dict" not in res


def test_ge_stores_urn():
    urn = "urn:great_expectations:stores:my_store:mymetric:kw=param"
    res = ge_urn.parseString(urn)

    assert res["urn_type"] == "stores"
    assert res["store_name"] == "my_store"
    assert res["metric_name"] == "mymetric"
    kwargs_dict = parse_qs(res["metric_kwargs"])
    assert kwargs_dict == {
        "kw": ["param"],
    }

    # No kwargs is ok
    urn = "urn:great_expectations:stores:my_store:mymetric"
    res = ge_urn.parseString(urn)

    assert res["urn_type"] == "stores"
    assert res["store_name"] == "my_store"
    assert res["metric_name"] == "mymetric"
    assert "metric_kwargs" not in res


def test_invalid_urn():
    # Must start with "urn:great_expectations"
    with pytest.raises(ParseException) as e:
        ge_urn.parseString("not_a_ge_urn")
    assert "not_a_ge_urn" in e.value.line

    # Must have one of the recognized types
    with pytest.raises(ParseException) as e:
        ge_urn.parseString("urn:great_expectations:foo:bar:baz:bin:barg")
    assert "urn:great_expectations:foo:bar:baz:bin:barg" in e.value.line

    # Cannot have too many parts
    with pytest.raises(ParseException) as e:
        ge_urn.parseString(
            "urn:great_expectations:validations:foo:bar:baz:bin:barg:boo"
        )
    assert "urn:great_expectations:validations:foo:bar:baz:bin:barg:boo" in e.value.line



import jsonschema
import mock
import pytest

from great_expectations.core.usage_statistics.schemas import (
    anonymized_batch_schema,
    anonymized_datasource_schema,
    cli_new_ds_choice_payload,
    datasource_sqlalchemy_connect_payload,
    empty_payload_schema,
    init_payload_schema,
    save_or_edit_expectation_suite_payload_schema,
    usage_statistics_record_schema,
)
from tests.integration.usage_statistics.test_usage_statistics_messages import (
    valid_usage_statistics_messages,
)


def test_comprehensive_list_of_messages():
    """Ensure that we have a comprehensive set of tests for known messages, by
    forcing a manual update to this list when a message is added or removed, and
    reminding the developer to add or remove the associate test."""
    valid_message_list = list(valid_usage_statistics_messages.keys())
    # NOTE: If you are changing the expected valid message list below, you need
    # to also update one or more tests below!

    assert valid_message_list == [
        "data_context.__init__",
        "data_asset.validate",
        "data_context.add_datasource",
        "data_context.build_data_docs",
        "data_context.open_data_docs",
        "data_context.save_expectation_suite",
        "datasource.sqlalchemy.connect",
        "cli.init.create",
        "cli.checkpoint.list",
        "cli.datasource.list",
        "cli.datasource.new",
        "cli.datasource.profile",
        "cli.docs.build",
        "cli.docs.clean",
        "cli.docs.list",
        "cli.new_ds_choice",
        "cli.project.check_config",
        "cli.store.list",
        "cli.suite.demo",
        "cli.suite.edit",
        "cli.suite.list",
        "cli.suite.new",
        "cli.suite.scaffold",
        "cli.validation_operator.list",
        "cli.validation_operator.run",
    ]


def test_init_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["data_context.__init__"][0],
        usage_statistics_record_schema,
    )
    # non-empty payload
    jsonschema.validate(
        valid_usage_statistics_messages["data_context.__init__"][0]["event_payload"],
        init_payload_schema,
    )


def test_data_asset_validate_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["data_asset.validate"][0],
        usage_statistics_record_schema,
    )
    # non-empty payload
    jsonschema.validate(
        valid_usage_statistics_messages["data_asset.validate"][0]["event_payload"],
        anonymized_batch_schema,
    )


def test_data_context_add_datasource_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["data_context.add_datasource"][0],
        usage_statistics_record_schema,
    )
    # non-empty payload
    jsonschema.validate(
        valid_usage_statistics_messages["data_context.add_datasource"][0][
            "event_payload"
        ],
        anonymized_datasource_schema,
    )


def test_data_context_save_expectation_suite_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["data_context.save_expectation_suite"][0],
        usage_statistics_record_schema,
    )
    # non-empty payload
    jsonschema.validate(
        valid_usage_statistics_messages["data_context.save_expectation_suite"][0][
            "event_payload"
        ],
        save_or_edit_expectation_suite_payload_schema,
    )


def test_datasource_sqlalchemy_connect_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["datasource.sqlalchemy.connect"][0],
        usage_statistics_record_schema,
    )
    jsonschema.validate(
        valid_usage_statistics_messages["datasource.sqlalchemy.connect"][0][
            "event_payload"
        ],
        datasource_sqlalchemy_connect_payload,
    )


def test_cli_new_ds_choice_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["data_asset.validate"][0],
        usage_statistics_record_schema,
    )
    # non-empty payload
    jsonschema.validate(
        valid_usage_statistics_messages["cli.new_ds_choice"][0]["event_payload"],
        cli_new_ds_choice_payload,
    )


def test_cli_suite_edit_message():
    # record itself
    jsonschema.validate(
        valid_usage_statistics_messages["cli.suite.edit"][0],
        usage_statistics_record_schema,
    )
    jsonschema.validate(
        valid_usage_statistics_messages["cli.suite.edit"][0]["event_payload"],
        save_or_edit_expectation_suite_payload_schema,
    )


def test_usage_stats_empty_payload_messages():
    usage_stats_records_messages = [
        "data_context.build_data_docs",
        "data_context.open_data_docs",
        "cli.suite.demo",
        "cli.suite.list",
        "cli.suite.new",
        "cli.suite.scaffold",
        "cli.store.list",
        "cli.project.check_config",
        "cli.validation_operator.list",
        "cli.validation_operator.run",
        "cli.docs.list",
        "cli.docs.build",
        "cli.datasource.list",
        "cli.datasource.new",
        "cli.init.create",
    ]
    for message in usage_stats_records_messages:
        jsonschema.validate(
            valid_usage_statistics_messages[message][0], usage_statistics_record_schema,
        )
        jsonschema.validate(
            valid_usage_statistics_messages[message][0]["event_payload"],
            empty_payload_schema,
        )



import pytest

from great_expectations.core import (
    ExpectationConfiguration,
    ExpectationValidationResult,
)
from great_expectations.data_asset import DataAsset
from great_expectations.dataset import MetaPandasDataset, PandasDataset


class ExpectationOnlyDataAsset(DataAsset):
    @DataAsset.expectation([])
    def no_op_expectation(
        self, result_format=None, include_config=True, catch_exceptions=None, meta=None
    ):
        return {"success": True}

    @DataAsset.expectation(["value"])
    def no_op_value_expectation(
        self,
        value=None,
        result_format=None,
        include_config=True,
        catch_exceptions=None,
        meta=None,
    ):
        return {"success": True}

    @DataAsset.expectation([])
    def exception_expectation(
        self, result_format=None, include_config=True, catch_exceptions=None, meta=None
    ):
        raise ValueError("Gotcha!")


def test_expectation_decorator_build_config():
    eds = ExpectationOnlyDataAsset()
    eds.no_op_expectation()
    eds.no_op_value_expectation("a")

    config = eds.get_expectation_suite()
    assert (
        ExpectationConfiguration(expectation_type="no_op_expectation", kwargs={})
        == config.expectations[0]
    )

    assert (
        ExpectationConfiguration(
            expectation_type="no_op_value_expectation", kwargs={"value": "a"},
        )
        == config.expectations[1]
    )


def test_expectation_decorator_include_config():
    eds = ExpectationOnlyDataAsset()
    out = eds.no_op_value_expectation("a", include_config=True)

    assert (
        ExpectationConfiguration(
            expectation_type="no_op_value_expectation",
            kwargs={"value": "a", "result_format": "BASIC"},
        )
        == out.expectation_config
    )


def test_expectation_decorator_meta():
    metadata = {"meta_key": "meta_value"}
    eds = ExpectationOnlyDataAsset()
    out = eds.no_op_value_expectation("a", meta=metadata)
    config = eds.get_expectation_suite()

    assert (
        ExpectationValidationResult(
            success=True, meta=metadata, expectation_config=config.expectations[0]
        )
        == out
    )

    assert (
        ExpectationConfiguration(
            expectation_type="no_op_value_expectation",
            kwargs={"value": "a"},
            meta=metadata,
        )
        == config.expectations[0]
    )


def test_expectation_decorator_catch_exceptions():
    eds = ExpectationOnlyDataAsset()

    # Confirm that we would raise an error without catching exceptions
    with pytest.raises(ValueError):
        eds.exception_expectation(catch_exceptions=False)

    # Catch exceptions and validate results
    out = eds.exception_expectation(catch_exceptions=True)
    assert out.exception_info["raised_exception"] is True

    # Check only the first and last line of the traceback, since formatting can be platform dependent.
    assert (
        "Traceback (most recent call last):"
        == out.exception_info["exception_traceback"].split("\n")[0]
    )
    assert (
        "ValueError: Gotcha!"
        == out.exception_info["exception_traceback"].split("\n")[-2]
    )


def test_pandas_column_map_decorator_partial_exception_counts():
    df = PandasDataset({"a": [0, 1, 2, 3, 4]})
    out = df.expect_column_values_to_be_between(
        "a",
        3,
        4,
        result_format={"result_format": "COMPLETE", "partial_unexpected_count": 1},
    )

    assert 1 == len(out.result["partial_unexpected_counts"])
    assert 3 == len(out.result["unexpected_list"])


def test_column_map_expectation_decorator():

    # Create a new CustomPandasDataset to
    # (1) demonstrate that custom subclassing works, and
    # (2) Test expectation business logic without dependencies on any other functions.
    class CustomPandasDataset(PandasDataset):
        @MetaPandasDataset.column_map_expectation
        def expect_column_values_to_be_odd(self, column):
            return column.map(lambda x: x % 2)

        @MetaPandasDataset.column_map_expectation
        def expectation_that_crashes_on_sixes(self, column):
            return column.map(lambda x: (x - 6) / 0 != "duck")

    df = CustomPandasDataset(
        {
            "all_odd": [1, 3, 5, 5, 5, 7, 9, 9, 9, 11],
            "mostly_odd": [1, 3, 5, 7, 9, 2, 4, 1, 3, 5],
            "all_even": [2, 4, 4, 6, 6, 6, 8, 8, 8, 8],
            "odd_missing": [1, 3, 5, None, None, None, None, 1, 3, None],
            "mixed_missing": [1, 3, 5, None, None, 2, 4, 1, 3, None],
            "all_missing": [None, None, None, None, None, None, None, None, None, None],
        }
    )
    df.set_default_expectation_argument("result_format", "COMPLETE")
    df.set_default_expectation_argument("include_config", False)

    assert df.expect_column_values_to_be_odd("all_odd") == ExpectationValidationResult(
        result={
            "element_count": 10,
            "missing_count": 0,
            "missing_percent": 0.0,
            "partial_unexpected_counts": [],
            "partial_unexpected_index_list": [],
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_index_list": [],
            "unexpected_list": [],
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
        },
        success=True,
    )

    assert df.expect_column_values_to_be_odd(
        "all_missing"
    ) == ExpectationValidationResult(
        result={
            "element_count": 10,
            "missing_count": 10,
            "missing_percent": 100.0,
            "partial_unexpected_counts": [],
            "partial_unexpected_index_list": [],
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_index_list": [],
            "unexpected_list": [],
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": None,
        },
        success=True,
    )

    assert df.expect_column_values_to_be_odd(
        "odd_missing"
    ) == ExpectationValidationResult(
        result={
            "element_count": 10,
            "missing_count": 5,
            "missing_percent": 50.0,
            "partial_unexpected_counts": [],
            "partial_unexpected_index_list": [],
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_index_list": [],
            "unexpected_list": [],
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
        },
        success=True,
    )

    assert df.expect_column_values_to_be_odd(
        "mixed_missing"
    ) == ExpectationValidationResult(
        result={
            "element_count": 10,
            "missing_count": 3,
            "missing_percent": 30.0,
            "partial_unexpected_counts": [
                {"value": 2.0, "count": 1},
                {"value": 4.0, "count": 1},
            ],
            "partial_unexpected_index_list": [5, 6],
            "partial_unexpected_list": [2.0, 4.0],
            "unexpected_count": 2,
            "unexpected_index_list": [5, 6],
            "unexpected_list": [2, 4],
            "unexpected_percent": 20.0,
            "unexpected_percent_nonmissing": (2 / 7 * 100),
        },
        success=False,
    )

    assert df.expect_column_values_to_be_odd(
        "mostly_odd"
    ) == ExpectationValidationResult(
        result={
            "element_count": 10,
            "missing_count": 0,
            "missing_percent": 0,
            "partial_unexpected_counts": [
                {"value": 2.0, "count": 1},
                {"value": 4.0, "count": 1},
            ],
            "partial_unexpected_index_list": [5, 6],
            "partial_unexpected_list": [2.0, 4.0],
            "unexpected_count": 2,
            "unexpected_index_list": [5, 6],
            "unexpected_list": [2, 4],
            "unexpected_percent": 20.0,
            "unexpected_percent_nonmissing": 20.0,
        },
        success=False,
    )

    assert df.expect_column_values_to_be_odd(
        "mostly_odd", mostly=0.6
    ) == ExpectationValidationResult(
        result={
            "element_count": 10,
            "missing_count": 0,
            "missing_percent": 0,
            "partial_unexpected_counts": [
                {"value": 2.0, "count": 1},
                {"value": 4.0, "count": 1},
            ],
            "partial_unexpected_index_list": [5, 6],
            "partial_unexpected_list": [2.0, 4.0],
            "unexpected_count": 2,
            "unexpected_index_list": [5, 6],
            "unexpected_list": [2, 4],
            "unexpected_percent": 20.0,
            "unexpected_percent_nonmissing": 20.0,
        },
        success=True,
    )

    assert df.expect_column_values_to_be_odd(
        "mostly_odd", result_format="BOOLEAN_ONLY"
    ) == ExpectationValidationResult(success=False)

    df.default_expectation_args["result_format"] = "BOOLEAN_ONLY"

    assert df.expect_column_values_to_be_odd(
        "mostly_odd"
    ) == ExpectationValidationResult(success=False)

    df.default_expectation_args["result_format"] = "BASIC"

    assert df.expect_column_values_to_be_odd(
        "mostly_odd", include_config=True
    ) == ExpectationValidationResult(
        expectation_config=ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_odd",
            kwargs={"column": "mostly_odd", "result_format": "BASIC"},
        ),
        result={
            "element_count": 10,
            "missing_count": 0,
            "missing_percent": 0,
            "partial_unexpected_list": [2, 4],
            "unexpected_count": 2,
            "unexpected_percent": 20.0,
            "unexpected_percent_nonmissing": 20.0,
        },
        success=False,
    )


def test_column_aggregate_expectation_decorator():
    # Create a new CustomPandasDataset to
    # (1) demonstrate that custom subclassing works, and
    # (2) Test expectation business logic without dependencies on any other functions.
    class CustomPandasDataset(PandasDataset):
        @PandasDataset.column_aggregate_expectation
        def expect_column_median_to_be_odd(self, column):
            median = self.get_column_median(column)
            return {"success": median % 2, "result": {"observed_value": median}}

    df = CustomPandasDataset(
        {
            "all_odd": [1, 3, 5, 7, 9],
            "all_even": [2, 4, 6, 8, 10],
            "odd_missing": [1, 3, 5, None, None],
            "mixed_missing": [1, 2, None, None, 6],
            "mixed_missing_2": [1, 3, None, None, 6],
            "all_missing": [None, None, None, None, None,],
        }
    )
    df.set_default_expectation_argument("result_format", "COMPLETE")
    df.set_default_expectation_argument("include_config", False)

    assert df.expect_column_median_to_be_odd("all_odd") == ExpectationValidationResult(
        result={
            "observed_value": 5,
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0,
        },
        success=True,
    )

    assert df.expect_column_median_to_be_odd("all_even") == ExpectationValidationResult(
        result={
            "observed_value": 6,
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0,
        },
        success=False,
    )

    assert df.expect_column_median_to_be_odd(
        "all_even", result_format="SUMMARY"
    ) == ExpectationValidationResult(
        result={
            "observed_value": 6,
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0,
        },
        success=False,
    )

    assert df.expect_column_median_to_be_odd(
        "all_even", result_format="BOOLEAN_ONLY"
    ) == ExpectationValidationResult(success=False)

    df.default_expectation_args["result_format"] = "BOOLEAN_ONLY"
    assert df.expect_column_median_to_be_odd("all_even") == ExpectationValidationResult(
        success=False
    )

    assert df.expect_column_median_to_be_odd(
        "all_even", result_format="BASIC"
    ) == ExpectationValidationResult(
        result={
            "observed_value": 6,
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0,
        },
        success=False,
    )


def test_column_pair_map_expectation_decorator():
    # Create a new CustomPandasDataset to
    # (1) Demonstrate that custom subclassing works, and
    # (2) Test expectation business logic without dependencies on any other functions.
    class CustomPandasDataset(PandasDataset):
        @PandasDataset.column_pair_map_expectation
        def expect_column_pair_values_to_be_different(
            self,
            column_A,
            column_B,
            keep_missing="either",
            output_format=None,
            include_config=True,
            catch_exceptions=None,
        ):
            return column_A != column_B

    df = CustomPandasDataset(
        {
            "all_odd": [1, 3, 5, 7, 9],
            "all_even": [2, 4, 6, 8, 10],
            "odd_missing": [1, 3, 5, None, None],
            "mixed_missing": [1, 2, None, None, 6],
            "mixed_missing_2": [1, 3, None, None, 6],
            "all_missing": [None, None, None, None, None,],
        }
    )
    df.set_default_expectation_argument("result_format", "COMPLETE")
    df.set_default_expectation_argument("include_config", False)

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "all_even"
    ) == ExpectationValidationResult(
        success=True,
        result={
            "element_count": 5,
            "missing_count": 0,
            "unexpected_count": 0,
            "missing_percent": 0.0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
            "unexpected_list": [],
            "unexpected_index_list": [],
            "partial_unexpected_list": [],
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
        },
    )

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "all_even", ignore_row_if="both_values_are_missing",
    ) == ExpectationValidationResult(
        success=True,
        result={
            "element_count": 5,
            "missing_count": 0,
            "unexpected_count": 0,
            "missing_percent": 0.0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
            "unexpected_list": [],
            "unexpected_index_list": [],
            "partial_unexpected_list": [],
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
        },
    )

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "odd_missing"
    ) == ExpectationValidationResult(
        success=False,
        result={
            "element_count": 5,
            "missing_count": 0,
            "unexpected_count": 3,
            "missing_percent": 0.0,
            "unexpected_percent": 60.0,
            "unexpected_percent_nonmissing": 60.0,
            "unexpected_list": [(1, 1.0), (3, 3.0), (5, 5.0)],
            "unexpected_index_list": [0, 1, 2],
            "partial_unexpected_list": [(1, 1.0), (3, 3.0), (5, 5.0)],
            "partial_unexpected_index_list": [0, 1, 2],
            "partial_unexpected_counts": [
                {"count": 1, "value": (1, 1.0)},
                {"count": 1, "value": (3, 3.0)},
                {"count": 1, "value": (5, 5.0)},
            ],
        },
    )

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "odd_missing", ignore_row_if="both_values_are_missing"
    ) == ExpectationValidationResult(
        success=False,
        result={
            "element_count": 5,
            "missing_count": 0,
            "unexpected_count": 3,
            "missing_percent": 0.0,
            "unexpected_percent": 60.0,
            "unexpected_percent_nonmissing": 60.0,
            "unexpected_list": [(1, 1.0), (3, 3.0), (5, 5.0)],
            "unexpected_index_list": [0, 1, 2],
            "partial_unexpected_list": [(1, 1.0), (3, 3.0), (5, 5.0)],
            "partial_unexpected_index_list": [0, 1, 2],
            "partial_unexpected_counts": [
                {"count": 1, "value": (1, 1.0)},
                {"count": 1, "value": (3, 3.0)},
                {"count": 1, "value": (5, 5.0)},
            ],
        },
    )

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "odd_missing", ignore_row_if="either_value_is_missing"
    ) == ExpectationValidationResult(
        success=False,
        result={
            "element_count": 5,
            "missing_count": 2,
            "unexpected_count": 3,
            "missing_percent": 40.0,
            "unexpected_percent": 60.0,
            "unexpected_percent_nonmissing": 100.0,
            "unexpected_list": [(1, 1.0), (3, 3.0), (5, 5.0)],
            "unexpected_index_list": [0, 1, 2],
            "partial_unexpected_list": [(1, 1.0), (3, 3.0), (5, 5.0)],
            "partial_unexpected_index_list": [0, 1, 2],
            "partial_unexpected_counts": [
                {"count": 1, "value": (1, 1.0)},
                {"count": 1, "value": (3, 3.0)},
                {"count": 1, "value": (5, 5.0)},
            ],
        },
    )

    with pytest.raises(ValueError):
        df.expect_column_pair_values_to_be_different(
            "all_odd", "odd_missing", ignore_row_if="blahblahblah"
        )

    # Test SUMMARY, BASIC, and BOOLEAN_ONLY output_formats
    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "all_even", result_format="SUMMARY"
    ) == ExpectationValidationResult(
        success=True,
        result={
            "element_count": 5,
            "missing_count": 0,
            "unexpected_count": 0,
            "missing_percent": 0.0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
            "partial_unexpected_list": [],
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
        },
    )

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "all_even", result_format="BASIC"
    ) == ExpectationValidationResult(
        success=True,
        result={
            "element_count": 5,
            "missing_count": 0,
            "unexpected_count": 0,
            "missing_percent": 0.0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
            "partial_unexpected_list": [],
        },
    )

    assert df.expect_column_pair_values_to_be_different(
        "all_odd", "all_even", result_format="BOOLEAN_ONLY"
    ) == ExpectationValidationResult(success=True)



import configparser
import os
import shutil
from copy import deepcopy

import jsonschema
import mock
import pytest

from great_expectations.core.usage_statistics.schemas import (
    anonymized_datasource_schema,
    cli_new_ds_choice_payload,
    init_payload_schema,
    save_or_edit_expectation_suite_payload_schema,
    usage_statistics_record_schema,
)
from great_expectations.core.usage_statistics.usage_statistics import (
    add_datasource_usage_statistics,
    run_validation_operator_usage_statistics,
)
from great_expectations.data_context import BaseDataContext, DataContext
from great_expectations.data_context.types.base import DataContextConfig
from great_expectations.data_context.util import file_relative_path
from tests.integration.usage_statistics.test_integration_usage_statistics import (
    USAGE_STATISTICS_QA_URL,
)


@pytest.fixture
def in_memory_data_context_config_usage_stats_enabled():
    return DataContextConfig(
        **{
            "commented_map": {},
            "config_version": 2,
            "plugins_directory": None,
            "evaluation_parameter_store_name": "evaluation_parameter_store",
            "validations_store_name": "validations_store",
            "expectations_store_name": "expectations_store",
            "config_variables_file_path": None,
            "datasources": {},
            "stores": {
                "expectations_store": {"class_name": "ExpectationsStore",},
                "validations_store": {"class_name": "ValidationsStore",},
                "evaluation_parameter_store": {
                    "class_name": "EvaluationParameterStore",
                },
            },
            "data_docs_sites": {},
            "validation_operators": {
                "default": {
                    "class_name": "ActionListValidationOperator",
                    "action_list": [],
                }
            },
            "anonymous_usage_statistics": {
                "enabled": True,
                "data_context_id": "00000000-0000-0000-0000-000000000001",
                "usage_statistics_url": USAGE_STATISTICS_QA_URL,
            },
        }
    )


def test_consistent_name_anonymization(
    in_memory_data_context_config_usage_stats_enabled, monkeypatch
):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    context = BaseDataContext(in_memory_data_context_config_usage_stats_enabled)
    assert context.data_context_id == "00000000-0000-0000-0000-000000000001"
    payload = run_validation_operator_usage_statistics(
        context,
        "action_list_operator",
        assets_to_validate=[
            ({"__fake_batch_kwargs": "mydatasource"}, "__fake_expectation_suite_name")
        ],
        run_id="foo",
    )
    # For a *specific* data_context_id, all names will be consistently anonymized
    assert payload["anonymized_operator_name"] == "e079c942d946b823312054118b3b6ef4"


def test_opt_out_environment_variable(
    in_memory_data_context_config_usage_stats_enabled, monkeypatch
):
    """Set the env variable GE_USAGE_STATS value to any of the following: FALSE, False, false, 0"""
    monkeypatch.setenv("GE_USAGE_STATS", "False")
    assert (
        in_memory_data_context_config_usage_stats_enabled.anonymous_usage_statistics.enabled
        is True
    )
    context = BaseDataContext(in_memory_data_context_config_usage_stats_enabled)
    project_config = context._project_config
    assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_etc(
    in_memory_data_context_config_usage_stats_enabled, tmp_path_factory, monkeypatch
):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    for false_string in ["False", "false", "f", "FALSE"]:
        disabled_config = configparser.ConfigParser()
        disabled_config["anonymous_usage_statistics"] = {"enabled": false_string}

        with open(
            os.path.join(etc_config_dir, "great_expectations.conf"), "w"
        ) as configfile:
            disabled_config.write(configfile)

        with mock.patch(
            "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
            config_dirs,
        ):
            assert (
                in_memory_data_context_config_usage_stats_enabled.anonymous_usage_statistics.enabled
                is True
            )
            context = BaseDataContext(
                deepcopy(in_memory_data_context_config_usage_stats_enabled)
            )
            project_config = context._project_config
            assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_home_folder(
    in_memory_data_context_config_usage_stats_enabled, tmp_path_factory, monkeypatch
):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    enabled_config = configparser.ConfigParser()
    enabled_config["anonymous_usage_statistics"] = {"enabled": "True"}

    for false_string in ["False", "false", "f", "FALSE"]:
        disabled_config = configparser.ConfigParser()
        disabled_config["anonymous_usage_statistics"] = {"enabled": false_string}

        with open(
            os.path.join(home_config_dir, "great_expectations.conf"), "w"
        ) as configfile:
            disabled_config.write(configfile)

        with mock.patch(
            "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
            config_dirs,
        ):
            assert (
                in_memory_data_context_config_usage_stats_enabled.anonymous_usage_statistics.enabled
                is True
            )
            context = BaseDataContext(
                deepcopy(in_memory_data_context_config_usage_stats_enabled)
            )
            project_config = context._project_config
            assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_yml(tmp_path_factory, monkeypatch):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    project_path = str(tmp_path_factory.mktemp("data_context"))
    context_path = os.path.join(project_path, "great_expectations")
    os.makedirs(context_path, exist_ok=True)
    fixture_dir = file_relative_path(__file__, "../../test_fixtures")

    shutil.copy(
        os.path.join(
            fixture_dir, "great_expectations_basic_with_usage_stats_disabled.yml"
        ),
        str(os.path.join(context_path, "great_expectations.yml")),
    )

    assert (
        DataContext(
            context_root_dir=context_path
        )._project_config.anonymous_usage_statistics.enabled
        is False
    )


# Test precedence: environment variable > home folder > /etc > yml
def test_opt_out_env_var_overrides_home_folder(
    in_memory_data_context_config_usage_stats_enabled, tmp_path_factory, monkeypatch
):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    enabled_config = configparser.ConfigParser()
    enabled_config["anonymous_usage_statistics"] = {"enabled": "True"}

    with open(
        os.path.join(home_config_dir, "great_expectations.conf"), "w"
    ) as configfile:
        enabled_config.write(configfile)

    monkeypatch.setenv("GE_USAGE_STATS", "False")

    with mock.patch(
        "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
        config_dirs,
    ):
        assert (
            in_memory_data_context_config_usage_stats_enabled.anonymous_usage_statistics.enabled
            is True
        )
        context = BaseDataContext(in_memory_data_context_config_usage_stats_enabled)
        project_config = context._project_config
        assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_env_var_overrides_etc(
    in_memory_data_context_config_usage_stats_enabled, tmp_path_factory, monkeypatch
):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    enabled_config = configparser.ConfigParser()
    enabled_config["anonymous_usage_statistics"] = {"enabled": "True"}

    with open(
        os.path.join(etc_config_dir, "great_expectations.conf"), "w"
    ) as configfile:
        enabled_config.write(configfile)

    monkeypatch.setenv("GE_USAGE_STATS", "False")

    with mock.patch(
        "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
        config_dirs,
    ):
        assert (
            in_memory_data_context_config_usage_stats_enabled.anonymous_usage_statistics.enabled
            is True
        )
        context = BaseDataContext(in_memory_data_context_config_usage_stats_enabled)
        project_config = context._project_config
        assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_env_var_overrides_yml(tmp_path_factory, monkeypatch):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    project_path = str(tmp_path_factory.mktemp("data_context"))
    context_path = os.path.join(project_path, "great_expectations")
    os.makedirs(context_path, exist_ok=True)
    fixture_dir = file_relative_path(__file__, "../../test_fixtures")

    shutil.copy(
        os.path.join(
            fixture_dir, "great_expectations_basic_with_usage_stats_enabled.yml"
        ),
        str(os.path.join(context_path, "great_expectations.yml")),
    )

    assert (
        DataContext(
            context_root_dir=context_path
        )._project_config.anonymous_usage_statistics.enabled
        is True
    )

    monkeypatch.setenv("GE_USAGE_STATS", "False")
    context = DataContext(context_root_dir=context_path)
    project_config = context._project_config
    assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_home_folder_overrides_etc(
    in_memory_data_context_config_usage_stats_enabled, tmp_path_factory, monkeypatch
):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    enabled_config = configparser.ConfigParser()
    enabled_config["anonymous_usage_statistics"] = {"enabled": "True"}

    disabled_config = configparser.ConfigParser()
    disabled_config["anonymous_usage_statistics"] = {"enabled": "False"}

    with open(
        os.path.join(home_config_dir, "great_expectations.conf"), "w"
    ) as configfile:
        disabled_config.write(configfile)
    with open(
        os.path.join(etc_config_dir, "great_expectations.conf"), "w"
    ) as configfile:
        enabled_config.write(configfile)

    with mock.patch(
        "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
        config_dirs,
    ):
        assert (
            in_memory_data_context_config_usage_stats_enabled.anonymous_usage_statistics.enabled
            is True
        )
        context = BaseDataContext(in_memory_data_context_config_usage_stats_enabled)
        project_config = context._project_config
        assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_home_folder_overrides_yml(tmp_path_factory, monkeypatch):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    disabled_config = configparser.ConfigParser()
    disabled_config["anonymous_usage_statistics"] = {"enabled": "False"}

    with open(
        os.path.join(home_config_dir, "great_expectations.conf"), "w"
    ) as configfile:
        disabled_config.write(configfile)

    project_path = str(tmp_path_factory.mktemp("data_context"))
    context_path = os.path.join(project_path, "great_expectations")
    os.makedirs(context_path, exist_ok=True)
    fixture_dir = file_relative_path(__file__, "../../test_fixtures")

    shutil.copy(
        os.path.join(
            fixture_dir, "great_expectations_basic_with_usage_stats_enabled.yml"
        ),
        str(os.path.join(context_path, "great_expectations.yml")),
    )

    assert (
        DataContext(
            context_root_dir=context_path
        )._project_config.anonymous_usage_statistics.enabled
        is True
    )

    with mock.patch(
        "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
        config_dirs,
    ):
        context = DataContext(context_root_dir=context_path)
        project_config = context._project_config
        assert project_config.anonymous_usage_statistics.enabled is False


def test_opt_out_etc_overrides_yml(tmp_path_factory, monkeypatch):
    monkeypatch.delenv(
        "GE_USAGE_STATS", raising=False
    )  # Undo the project-wide test default
    home_config_dir = tmp_path_factory.mktemp("home_dir")
    home_config_dir = str(home_config_dir)
    etc_config_dir = tmp_path_factory.mktemp("etc")
    etc_config_dir = str(etc_config_dir)
    config_dirs = [home_config_dir, etc_config_dir]
    config_dirs = [
        os.path.join(config_dir, "great_expectations.conf")
        for config_dir in config_dirs
    ]

    disabled_config = configparser.ConfigParser()
    disabled_config["anonymous_usage_statistics"] = {"enabled": "False"}

    with open(
        os.path.join(etc_config_dir, "great_expectations.conf"), "w"
    ) as configfile:
        disabled_config.write(configfile)

    project_path = str(tmp_path_factory.mktemp("data_context"))
    context_path = os.path.join(project_path, "great_expectations")
    os.makedirs(context_path, exist_ok=True)
    fixture_dir = file_relative_path(__file__, "../../test_fixtures")

    shutil.copy(
        os.path.join(
            fixture_dir, "great_expectations_basic_with_usage_stats_enabled.yml"
        ),
        str(os.path.join(context_path, "great_expectations.yml")),
    )

    assert (
        DataContext(
            context_root_dir=context_path
        )._project_config.anonymous_usage_statistics.enabled
        is True
    )

    with mock.patch(
        "great_expectations.data_context.BaseDataContext.GLOBAL_CONFIG_PATHS",
        config_dirs,
    ):
        context = DataContext(context_root_dir=context_path)
        project_config = context._project_config
        assert project_config.anonymous_usage_statistics.enabled is False



import decimal

import numpy as np
import pandas as pd
import pytest

import great_expectations as ge
from great_expectations.core import (
    ExpectationConfiguration,
    ExpectationSuite,
    ExpectationValidationResult,
    expectationSuiteSchema,
)
from great_expectations.exceptions import InvalidExpectationConfigurationError


def test_get_and_save_expectation_suite(tmp_path_factory):
    directory_name = str(
        tmp_path_factory.mktemp("test_get_and_save_expectation_config")
    )
    df = ge.dataset.PandasDataset(
        {"x": [1, 2, 4], "y": [1, 2, 5], "z": ["hello", "jello", "mello"],}
    )

    df.expect_column_values_to_be_in_set("x", [1, 2, 4])
    df.expect_column_values_to_be_in_set(
        "y", [1, 2, 4], catch_exceptions=True, include_config=True
    )
    df.expect_column_values_to_match_regex("z", "ello")

    ### First test set ###

    output_config = ExpectationSuite(
        expectations=[
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_be_in_set",
                kwargs={"column": "x", "value_set": [1, 2, 4]},
            ),
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_match_regex",
                kwargs={"column": "z", "regex": "ello"},
            ),
        ],
        expectation_suite_name="default",
        data_asset_type="Dataset",
        meta={"great_expectations_version": ge.__version__},
    )

    assert output_config == df.get_expectation_suite()

    df.save_expectation_suite(directory_name + "/temp1.json")
    with open(directory_name + "/temp1.json") as infile:
        loaded_config = expectationSuiteSchema.loads(infile.read())
    assert output_config == loaded_config

    ### Second test set ###

    output_config = ExpectationSuite(
        expectations=[
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_be_in_set",
                kwargs={"column": "x", "value_set": [1, 2, 4]},
            ),
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_be_in_set",
                kwargs={"column": "y", "value_set": [1, 2, 4]},
            ),
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_match_regex",
                kwargs={"column": "z", "regex": "ello"},
            ),
        ],
        expectation_suite_name="default",
        data_asset_type="Dataset",
        meta={"great_expectations_version": ge.__version__},
    )

    assert output_config == df.get_expectation_suite(discard_failed_expectations=False)
    df.save_expectation_suite(
        directory_name + "/temp2.json", discard_failed_expectations=False
    )
    with open(directory_name + "/temp2.json") as infile:
        loaded_suite = expectationSuiteSchema.loads(infile.read())
    assert output_config == loaded_suite

    ### Third test set ###

    output_config = ExpectationSuite(
        expectations=[
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_be_in_set",
                kwargs={
                    "column": "x",
                    "value_set": [1, 2, 4],
                    "result_format": "BASIC",
                },
            ),
            ExpectationConfiguration(
                expectation_type="expect_column_values_to_match_regex",
                kwargs={"column": "z", "regex": "ello", "result_format": "BASIC"},
            ),
        ],
        expectation_suite_name="default",
        data_asset_type="Dataset",
        meta={"great_expectations_version": ge.__version__},
    )
    assert output_config == df.get_expectation_suite(
        discard_result_format_kwargs=False,
        discard_include_config_kwargs=False,
        discard_catch_exceptions_kwargs=False,
    )

    df.save_expectation_suite(
        directory_name + "/temp3.json",
        discard_result_format_kwargs=False,
        discard_include_config_kwargs=False,
        discard_catch_exceptions_kwargs=False,
    )
    with open(directory_name + "/temp3.json") as infile:
        loaded_suite = expectationSuiteSchema.loads(infile.read())
    assert output_config == loaded_suite


def test_expectation_meta():
    df = ge.dataset.PandasDataset(
        {"x": [1, 2, 4], "y": [1, 2, 5], "z": ["hello", "jello", "mello"],}
    )
    result = df.expect_column_median_to_be_between(
        "x", 2, 2, meta={"notes": "This expectation is for lolz."}
    )
    k = 0
    assert True is result.success
    suite = df.get_expectation_suite()
    for expectation_config in suite.expectations:
        if expectation_config.expectation_type == "expect_column_median_to_be_between":
            k += 1
            assert {"notes": "This expectation is for lolz."} == expectation_config.meta
    assert 1 == k

    # This should raise an error because meta isn't serializable.
    with pytest.raises(InvalidExpectationConfigurationError) as exc:
        df.expect_column_values_to_be_increasing(
            "x", meta={"unserializable_content": np.complex(0, 0)}
        )
    assert "which cannot be serialized to json" in exc.value.message


def test_set_default_expectation_argument():
    df = ge.dataset.PandasDataset(
        {"x": [1, 2, 4], "y": [1, 2, 5], "z": ["hello", "jello", "mello"],}
    )

    assert {
        "include_config": True,
        "catch_exceptions": False,
        "result_format": "BASIC",
    } == df.get_default_expectation_arguments()

    df.set_default_expectation_argument("result_format", "SUMMARY")

    assert {
        "include_config": True,
        "catch_exceptions": False,
        "result_format": "SUMMARY",
    } == df.get_default_expectation_arguments()


def test_test_column_map_expectation_function():
    asset = ge.dataset.PandasDataset({"x": [1, 3, 5, 7, 9], "y": [1, 2, None, 7, 9],})

    def is_odd(
        self,
        column,
        mostly=None,
        result_format=None,
        include_config=True,
        catch_exceptions=None,
        meta=None,
    ):
        return column % 2 == 1

    assert asset.test_column_map_expectation_function(
        is_odd, column="x", include_config=False
    ) == ExpectationValidationResult(
        result={
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0,
            "unexpected_percent": 0.0,
            "partial_unexpected_list": [],
            "unexpected_percent_nonmissing": 0.0,
            "unexpected_count": 0,
        },
        success=True,
    )

    assert asset.test_column_map_expectation_function(
        is_odd, "x", result_format="BOOLEAN_ONLY", include_config=False
    ) == ExpectationValidationResult(success=True)

    assert asset.test_column_map_expectation_function(
        is_odd, column="y", result_format="BOOLEAN_ONLY", include_config=False
    ) == ExpectationValidationResult(success=False)

    assert asset.test_column_map_expectation_function(
        is_odd,
        column="y",
        result_format="BOOLEAN_ONLY",
        mostly=0.7,
        include_config=False,
    ) == ExpectationValidationResult(success=True)


def test_test_column_aggregate_expectation_function():
    asset = ge.dataset.PandasDataset({"x": [1, 3, 5, 7, 9], "y": [1, 2, None, 7, 9],})

    def expect_second_value_to_be(
        self,
        column,
        value,
        result_format=None,
        include_config=True,
        catch_exceptions=None,
        meta=None,
    ):
        return {
            "success": self[column].iloc[1] == value,
            "result": {"observed_value": self[column].iloc[1],},
        }

    assert asset.test_column_aggregate_expectation_function(
        expect_second_value_to_be, "x", 2, include_config=False
    ) == ExpectationValidationResult(
        result={
            "observed_value": 3,
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0.0,
        },
        success=False,
    )

    assert asset.test_column_aggregate_expectation_function(
        expect_second_value_to_be, column="x", value=3, include_config=False
    ) == ExpectationValidationResult(
        result={
            "observed_value": 3.0,
            "element_count": 5,
            "missing_count": 0,
            "missing_percent": 0.0,
        },
        success=True,
    )

    assert asset.test_column_aggregate_expectation_function(
        expect_second_value_to_be,
        "y",
        value=3,
        result_format="BOOLEAN_ONLY",
        include_config=False,
    ) == ExpectationValidationResult(success=False)

    assert asset.test_column_aggregate_expectation_function(
        expect_second_value_to_be,
        "y",
        2,
        result_format="BOOLEAN_ONLY",
        include_config=False,
    ) == ExpectationValidationResult(success=True)


def test_meta_version_warning():
    asset = ge.data_asset.DataAsset()

    with pytest.warns(UserWarning) as w:
        suite = ExpectationSuite(expectations=[], expectation_suite_name="test")
        # mangle the metadata
        suite.meta = {"foo": "bar"}
        out = asset.validate(expectation_suite=suite)
    assert (
        w[0].message.args[0]
        == "WARNING: No great_expectations version found in configuration object."
    )

    with pytest.warns(UserWarning) as w:
        suite = ExpectationSuite(
            expectations=[],
            expectation_suite_name="test",
            meta={"great_expectations_version": "0.0.0"},
        )
        # mangle the metadata
        suite.meta = {"great_expectations_version": "0.0.0"}
        out = asset.validate(expectation_suite=suite)
    assert (
        w[0].message.args[0]
        == "WARNING: This configuration object was built using version 0.0.0 of great_expectations, but is currently "
        "being validated by version %s." % ge.__version__
    )


def test_format_map_output():
    df = ge.dataset.PandasDataset({"x": list("abcdefghijklmnopqrstuvwxyz"),})

    ### Normal Test ###

    success = True
    element_count = 20
    nonnull_values = pd.Series(range(15))
    nonnull_count = 15
    boolean_mapped_success_values = pd.Series([True for i in range(15)])
    success_count = 15
    unexpected_list = []
    unexpected_index_list = []

    assert df._format_map_output(
        "BOOLEAN_ONLY",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {"success": True}

    assert df._format_map_output(
        "BASIC",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": True,
        "result": {
            "element_count": 20,
            "missing_count": 5,
            "missing_percent": 25.0,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
        },
    }

    assert df._format_map_output(
        "SUMMARY",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": True,
        "result": {
            "element_count": 20,
            "missing_count": 5,
            "missing_percent": 25.0,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
        },
    }

    assert df._format_map_output(
        "COMPLETE",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": True,
        "result": {
            "element_count": 20,
            "missing_count": 5,
            "missing_percent": 25.0,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": 0.0,
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
            "unexpected_list": [],
            "unexpected_index_list": [],
        },
    }

    ### Test the case where all elements are null ###

    success = True
    element_count = 20
    nonnull_values = pd.Series([])
    nonnull_count = 0
    boolean_mapped_success_values = pd.Series([])
    success_count = 0
    unexpected_list = []
    unexpected_index_list = []

    assert df._format_map_output(
        "BOOLEAN_ONLY",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {"success": True}

    assert df._format_map_output(
        "BASIC",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": True,
        "result": {
            "element_count": 20,
            "missing_count": 20,
            "missing_percent": 100,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": None,
        },
    }

    assert df._format_map_output(
        "SUMMARY",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": True,
        "result": {
            "element_count": 20,
            "missing_count": 20,
            "missing_percent": 100,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": None,
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
        },
    }

    assert df._format_map_output(
        "COMPLETE",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": True,
        "result": {
            "element_count": 20,
            "missing_count": 20,
            "missing_percent": 100,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": 0.0,
            "unexpected_percent_nonmissing": None,
            "partial_unexpected_index_list": [],
            "partial_unexpected_counts": [],
            "unexpected_list": [],
            "unexpected_index_list": [],
        },
    }

    ### Test the degenerate case where there are no elements ###

    success = False
    element_count = 0
    nonnull_values = pd.Series([])
    nonnull_count = 0
    boolean_mapped_success_values = pd.Series([])
    success_count = 0
    unexpected_list = []
    unexpected_index_list = []

    assert df._format_map_output(
        "BOOLEAN_ONLY",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {"success": False}

    assert df._format_map_output(
        "BASIC",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": False,
        "result": {
            "element_count": 0,
            "missing_count": 0,
            "missing_percent": None,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": None,
            "unexpected_percent_nonmissing": None,
        },
    }

    assert df._format_map_output(
        "SUMMARY",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": False,
        "result": {
            "element_count": 0,
            "missing_count": 0,
            "missing_percent": None,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": None,
            "unexpected_percent_nonmissing": None,
            "partial_unexpected_counts": [],
            "partial_unexpected_index_list": [],
        },
    }

    assert df._format_map_output(
        "COMPLETE",
        success,
        element_count,
        nonnull_count,
        len(unexpected_list),
        unexpected_list,
        unexpected_index_list,
    ) == {
        "success": False,
        "result": {
            "element_count": 0,
            "missing_count": 0,
            "missing_percent": None,
            "partial_unexpected_list": [],
            "unexpected_count": 0,
            "unexpected_percent": None,
            "unexpected_percent_nonmissing": None,
            "partial_unexpected_counts": [],
            "partial_unexpected_index_list": [],
            "unexpected_list": [],
            "unexpected_index_list": [],
        },
    }


def test_calc_map_expectation_success():
    df = ge.dataset.PandasDataset({"x": list("abcdefghijklmnopqrstuvwxyz")})
    assert df._calc_map_expectation_success(
        success_count=10, nonnull_count=10, mostly=None
    ) == (True, 1.0)

    assert df._calc_map_expectation_success(
        success_count=90, nonnull_count=100, mostly=0.9
    ) == (True, 0.9)

    assert df._calc_map_expectation_success(
        success_count=90, nonnull_count=100, mostly=0.8
    ) == (True, 0.9)

    assert df._calc_map_expectation_success(
        success_count=80, nonnull_count=100, mostly=0.9
    ) == (False, 0.8)

    assert df._calc_map_expectation_success(
        success_count=0, nonnull_count=0, mostly=None
    ) == (True, None)

    assert df._calc_map_expectation_success(
        success_count=0, nonnull_count=100, mostly=None
    ) == (False, 0.0)

    # NOTE - 20200229 - Alex/James:
    # We no longer allow DECIMAL explicitly, because we do not expect
    # decimal values in response to queries given potential problems
    # with comparison to a float-valued mostly parameter

    # assert df._calc_map_expectation_success(
    #     success_count=decimal.Decimal(80), nonnull_count=100, mostly=0.8
    # ) == (False, decimal.Decimal(80) / decimal.Decimal(100))
    with pytest.raises(ValueError) as exc:
        df._calc_map_expectation_success(
            success_count=decimal.Decimal(80), nonnull_count=100, mostly=0.8
        )
    assert "must not be a decimal; check your db configuration" in str(exc.value)

    assert df._calc_map_expectation_success(
        success_count=100, nonnull_count=100, mostly=0
    ) == (True, 1.0)

    assert df._calc_map_expectation_success(
        success_count=50, nonnull_count=100, mostly=0
    ) == (True, 0.5)

    assert df._calc_map_expectation_success(
        success_count=0, nonnull_count=100, mostly=0
    ) == (True, 0.0)


def test_discard_failing_expectations():
    df = ge.dataset.PandasDataset(
        {
            "A": [1, 2, 3, 4],
            "B": [5, 6, 7, 8],
            "C": ["a", "b", "c", "d"],
            "D": ["e", "f", "g", "h"],
        },
        profiler=ge.profile.ColumnsExistProfiler,
    )

    # Put some simple expectations on the data frame
    df.expect_column_values_to_be_in_set("A", [1, 2, 3, 4])
    df.expect_column_values_to_be_in_set("B", [5, 6, 7, 8])
    df.expect_column_values_to_be_in_set("C", ["a", "b", "c", "d"])
    df.expect_column_values_to_be_in_set("D", ["e", "f", "g", "h"])

    exp1 = [
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "A"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "B"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "C"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "D"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "A", "value_set": [1, 2, 3, 4]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "B", "value_set": [5, 6, 7, 8]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "C", "value_set": ["a", "b", "c", "d"]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "D", "value_set": ["e", "f", "g", "h"]},
        ),
    ]

    sub1 = df[:3]

    sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df[1:2]
    sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df[:-1]
    sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df[-1:]
    sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df[["A", "D"]]
    exp1 = [
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "A"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "D"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "A", "value_set": [1, 2, 3, 4]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "D", "value_set": ["e", "f", "g", "h"]},
        ),
    ]
    with pytest.warns(UserWarning, match=r"Removed \d expectations that were 'False'"):
        sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df[["A"]]
    exp1 = [
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "A"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "A", "value_set": [1, 2, 3, 4]},
        ),
    ]
    with pytest.warns(UserWarning, match=r"Removed \d expectations that were 'False'"):
        sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df.iloc[:3, 1:4]
    exp1 = [
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "B"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "C"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "D"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "B", "value_set": [5, 6, 7, 8]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "C", "value_set": ["a", "b", "c", "d"]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "D", "value_set": ["e", "f", "g", "h"]},
        ),
    ]
    with pytest.warns(UserWarning, match=r"Removed \d expectations that were 'False'"):
        sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1

    sub1 = df.loc[0:, "A":"B"]
    exp1 = [
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "A"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_to_exist", kwargs={"column": "B"}
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "A", "value_set": [1, 2, 3, 4]},
        ),
        ExpectationConfiguration(
            expectation_type="expect_column_values_to_be_in_set",
            kwargs={"column": "B", "value_set": [5, 6, 7, 8]},
        ),
    ]
    with pytest.warns(UserWarning, match=r"Removed \d expectations that were 'False'"):
        sub1.discard_failing_expectations()
    assert sub1.get_expectation_suite().expectations == exp1


def test_test_expectation_function():
    asset = ge.dataset.PandasDataset({"x": [1, 3, 5, 7, 9], "y": [1, 2, None, 7, 9],})
    asset_2 = ge.dataset.PandasDataset({"x": [1, 3, 5, 6, 9], "y": [1, 2, None, 6, 9],})

    def expect_dataframe_to_contain_7(self):
        return {"success": bool((self == 7).sum().sum() > 0)}

    assert asset.test_expectation_function(
        expect_dataframe_to_contain_7, include_config=False
    ) == ExpectationValidationResult(success=True)
    assert asset_2.test_expectation_function(
        expect_dataframe_to_contain_7, include_config=False
    ) == ExpectationValidationResult(success=False)



# -*- coding: utf-8 -*-
import json
import os
from collections import OrderedDict

import mistune
import pytest

from great_expectations.core import ExpectationConfiguration, ExpectationSuite
from great_expectations.data_context.util import file_relative_path
from great_expectations.render.renderer import (
    ExpectationSuitePageRenderer,
    ProfilingResultsPageRenderer,
    ValidationResultsPageRenderer,
)
from great_expectations.render.types import RenderedContent


def test_ExpectationSuitePageRenderer_render_expectation_suite_notes():
    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(expectation_suite_name="test", meta={"notes": "*hi*"})
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))
    assert RenderedContent.rendered_content_list_to_json(result.text) == [
        "This Expectation suite currently contains 0 total Expectations across 0 columns.",
        "*hi*",
    ]

    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(
            expectation_suite_name="test",
            meta={"notes": ["*alpha*", "_bravo_", "charlie"]},
        )
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))
    assert RenderedContent.rendered_content_list_to_json(result.text) == [
        "This Expectation suite currently contains 0 total Expectations across 0 columns.",
        "*alpha*",
        "_bravo_",
        "charlie",
    ]

    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(
            expectation_suite_name="test",
            meta={
                "notes": {
                    "format": "string",
                    "content": ["*alpha*", "_bravo_", "charlie"],
                }
            },
        )
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))
    assert RenderedContent.rendered_content_list_to_json(result.text) == [
        "This Expectation suite currently contains 0 total Expectations across 0 columns.",
        "*alpha*",
        "_bravo_",
        "charlie",
    ]

    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(
            expectation_suite_name="test",
            meta={"notes": {"format": "markdown", "content": "*alpha*"}},
        )
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))

    try:
        mistune.markdown("*test*")
        assert RenderedContent.rendered_content_list_to_json(result.text) == [
            "This Expectation suite currently contains 0 total Expectations across 0 columns.",
            {
                "content_block_type": "markdown",
                "styling": {"parent": {}},
                "markdown": "*alpha*",
            },
        ]
    except OSError:
        assert RenderedContent.rendered_content_list_to_json(result.text) == [
            "This Expectation suite currently contains 0 total Expectations across 0 columns.",
            "*alpha*",
        ]

    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(
            expectation_suite_name="test",
            meta={
                "notes": {
                    "format": "markdown",
                    "content": ["*alpha*", "_bravo_", "charlie"],
                }
            },
        )
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))

    try:
        mistune.markdown("*test*")
        assert RenderedContent.rendered_content_list_to_json(result.text) == [
            "This Expectation suite currently contains 0 total Expectations across 0 columns.",
            {
                "content_block_type": "markdown",
                "styling": {"parent": {}},
                "markdown": "*alpha*",
            },
            {
                "content_block_type": "markdown",
                "styling": {"parent": {}},
                "markdown": "_bravo_",
            },
            {
                "content_block_type": "markdown",
                "styling": {"parent": {}},
                "markdown": "charlie",
            },
        ]
    except OSError:
        assert RenderedContent.rendered_content_list_to_json(result.text) == [
            "This Expectation suite currently contains 0 total Expectations across 0 columns.",
            "*alpha*",
            "_bravo_",
            "charlie",
        ]


def test_expectation_summary_in_ExpectationSuitePageRenderer_render_expectation_suite_notes():
    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(expectation_suite_name="test", meta={}, expectations=None)
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))
    assert RenderedContent.rendered_content_list_to_json(result.text) == [
        "This Expectation suite currently contains 0 total Expectations across 0 columns."
    ]

    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(
            expectation_suite_name="test",
            meta={"notes": {"format": "markdown", "content": ["hi"]}},
        )
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text))

    try:
        mistune.markdown("*test*")
        assert RenderedContent.rendered_content_list_to_json(result.text) == [
            "This Expectation suite currently contains 0 total Expectations across 0 columns.",
            {
                "content_block_type": "markdown",
                "styling": {"parent": {}},
                "markdown": "hi",
            },
        ]
    except OSError:
        assert RenderedContent.rendered_content_list_to_json(result.text) == [
            "This Expectation suite currently contains 0 total Expectations across 0 columns.",
            "hi",
        ]

    result = ExpectationSuitePageRenderer._render_expectation_suite_notes(
        ExpectationSuite(
            expectation_suite_name="test",
            meta={},
            expectations=[
                ExpectationConfiguration(
                    expectation_type="expect_table_row_count_to_be_between",
                    kwargs={"min_value": 0, "max_value": None},
                ),
                ExpectationConfiguration(
                    expectation_type="expect_column_to_exist", kwargs={"column": "x"}
                ),
                ExpectationConfiguration(
                    expectation_type="expect_column_to_exist", kwargs={"column": "y"}
                ),
            ],
        )
    )
    # print(RenderedContent.rendered_content_list_to_json(result.text)[0])
    assert (
        RenderedContent.rendered_content_list_to_json(result.text)[0]
        == "This Expectation suite currently contains 3 total Expectations across 2 columns."
    )


def test_ProfilingResultsPageRenderer(titanic_profiled_evrs_1):
    document = ProfilingResultsPageRenderer().render(titanic_profiled_evrs_1)
    # print(document)
    # assert document == 0


def test_ValidationResultsPageRenderer_render_validation_header(
    titanic_profiled_evrs_1,
):
    validation_header = ValidationResultsPageRenderer._render_validation_header(
        titanic_profiled_evrs_1
    ).to_json_dict()

    expected_validation_header = {
        "content_block_type": "header",
        "styling": {
            "classes": ["col-12", "p-0"],
            "header": {"classes": ["alert", "alert-secondary"]},
        },
        "header": {
            "content_block_type": "string_template",
            "string_template": {
                "template": "Overview",
                "tag": "h5",
                "styling": {"classes": ["m-0"]},
            },
        },
        "subheader": {
            "content_block_type": "string_template",
            "string_template": {
                "template": "${suite_title} ${expectation_suite_name}\n${status_title} ${html_success_icon} ${success}",
                "params": {
                    "suite_title": "Expectation Suite:",
                    "status_title": "Status:",
                    "expectation_suite_name": "default",
                    "success": "Failed",
                    "html_success_icon": '<i class="fas fa-times text-danger" aria-hidden="true"></i>',
                },
                "styling": {
                    "params": {
                        "suite_title": {"classes": ["h6"]},
                        "status_title": {"classes": ["h6"]},
                        "expectation_suite_name": {
                            "tag": "a",
                            "attributes": {
                                "href": "../../../../expectations/default.html"
                            },
                        },
                    },
                    "classes": ["mb-0", "mt-1"],
                },
            },
        },
    }

    # print(validation_header)
    assert validation_header == expected_validation_header


def test_ValidationResultsPageRenderer_render_validation_info(titanic_profiled_evrs_1):
    validation_info = ValidationResultsPageRenderer._render_validation_info(
        titanic_profiled_evrs_1
    ).to_json_dict()
    print(validation_info)

    expected_validation_info = {
        "content_block_type": "table",
        "styling": {
            "classes": ["col-12", "table-responsive", "mt-1"],
            "body": {
                "classes": ["table", "table-sm"],
                "styles": {
                    "margin-bottom": "0.5rem !important",
                    "margin-top": "0.5rem !important",
                },
            },
        },
        "header": {
            "content_block_type": "string_template",
            "string_template": {
                "template": "Info",
                "tag": "h6",
                "styling": {"classes": ["m-0"]},
            },
        },
        "table": [
            ["Great Expectations Version", "0.9.7+17.g02805059.dirty"],
            ["Run Name", "20200322T170247.671855Z"],
            ["Run Time", "2020-03-22T17:02:47.671855Z"],
        ],
    }

    assert validation_info == expected_validation_info


def test_ValidationResultsPageRenderer_render_validation_statistics(
    titanic_profiled_evrs_1,
):
    validation_statistics = ValidationResultsPageRenderer._render_validation_statistics(
        titanic_profiled_evrs_1
    ).to_json_dict()
    print(validation_statistics)
    expected_validation_statistics = {
        "content_block_type": "table",
        "styling": {
            "classes": ["col-6", "table-responsive", "mt-1", "p-1"],
            "body": {
                "classes": ["table", "table-sm"],
                "styles": {
                    "margin-bottom": "0.5rem !important",
                    "margin-top": "0.5rem !important",
                },
            },
        },
        "header": {
            "content_block_type": "string_template",
            "string_template": {
                "template": "Statistics",
                "tag": "h6",
                "styling": {"classes": ["m-0"]},
            },
        },
        "table": [
            ["Evaluated Expectations", 51],
            ["Successful Expectations", 43],
            ["Unsuccessful Expectations", 8],
            ["Success Percent", "≈84.31%"],
        ],
    }

    assert validation_statistics == expected_validation_statistics


def test_ValidationResultsPageRenderer_render_nested_table_from_dict():
    batch_kwargs = {
        "path": "project_dir/project_path/data/titanic/Titanic.csv",
        "datasource": "Titanic",
        "reader_options": {"sep": None, "engine": "python"},
    }
    batch_kwargs_table = ValidationResultsPageRenderer._render_nested_table_from_dict(
        batch_kwargs, header="Batch Kwargs"
    ).to_json_dict()
    print(batch_kwargs_table)

    expected_batch_kwarg_table = {
        "content_block_type": "table",
        "styling": {
            "body": {
                "classes": ["table", "table-sm"],
                "styles": {
                    "margin-bottom": "0.5rem !important",
                    "margin-top": "0.5rem !important",
                },
            }
        },
        "header": {
            "content_block_type": "string_template",
            "string_template": {
                "template": "Batch Kwargs",
                "tag": "h6",
                "styling": {"classes": ["m-0"]},
            },
        },
        "table": [
            [
                {
                    "content_block_type": "string_template",
                    "styling": {"parent": {"classes": ["pr-3"]}},
                    "string_template": {
                        "template": "$value",
                        "params": {"value": "datasource"},
                        "styling": {"default": {"styles": {"word-break": "break-all"}}},
                    },
                },
                {
                    "content_block_type": "string_template",
                    "styling": {"parent": {"classes": []}},
                    "string_template": {
                        "template": "$value",
                        "params": {"value": "Titanic"},
                        "styling": {"default": {"styles": {"word-break": "break-all"}}},
                    },
                },
            ],
            [
                {
                    "content_block_type": "string_template",
                    "styling": {"parent": {"classes": ["pr-3"]}},
                    "string_template": {
                        "template": "$value",
                        "params": {"value": "path"},
                        "styling": {"default": {"styles": {"word-break": "break-all"}}},
                    },
                },
                {
                    "content_block_type": "string_template",
                    "styling": {"parent": {"classes": []}},
                    "string_template": {
                        "template": "$value",
                        "params": {
                            "value": "project_dir/project_path/data/titanic/Titanic.csv"
                        },
                        "styling": {"default": {"styles": {"word-break": "break-all"}}},
                    },
                },
            ],
            [
                {
                    "content_block_type": "string_template",
                    "styling": {"parent": {"classes": ["pr-3"]}},
                    "string_template": {
                        "template": "$value",
                        "params": {"value": "reader_options"},
                        "styling": {"default": {"styles": {"word-break": "break-all"}}},
                    },
                },
                {
                    "content_block_type": "table",
                    "styling": {
                        "classes": ["col-6", "table-responsive"],
                        "body": {"classes": ["table", "table-sm", "m-0"]},
                        "parent": {"classes": ["pt-0", "pl-0", "border-top-0"]},
                    },
                    "table": [
                        [
                            {
                                "content_block_type": "string_template",
                                "styling": {"parent": {"classes": ["pr-3"]}},
                                "string_template": {
                                    "template": "$value",
                                    "params": {"value": "engine"},
                                    "styling": {
                                        "default": {
                                            "styles": {"word-break": "break-all"}
                                        }
                                    },
                                },
                            },
                            {
                                "content_block_type": "string_template",
                                "styling": {"parent": {"classes": []}},
                                "string_template": {
                                    "template": "$value",
                                    "params": {"value": "python"},
                                    "styling": {
                                        "default": {
                                            "styles": {"word-break": "break-all"}
                                        }
                                    },
                                },
                            },
                        ],
                        [
                            {
                                "content_block_type": "string_template",
                                "styling": {"parent": {"classes": ["pr-3"]}},
                                "string_template": {
                                    "template": "$value",
                                    "params": {"value": "sep"},
                                    "styling": {
                                        "default": {
                                            "styles": {"word-break": "break-all"}
                                        }
                                    },
                                },
                            },
                            {
                                "content_block_type": "string_template",
                                "styling": {"parent": {"classes": []}},
                                "string_template": {
                                    "template": "$value",
                                    "params": {"value": "None"},
                                    "styling": {
                                        "default": {
                                            "styles": {"word-break": "break-all"}
                                        }
                                    },
                                },
                            },
                        ],
                    ],
                },
            ],
        ],
    }

    assert batch_kwargs_table == expected_batch_kwarg_table


@pytest.fixture()
def ValidationResultsPageRenderer_render_with_run_info_at_end():
    """
    Rendered validation results with run info at the end
    Returns:
        json string of rendered validation results
    """
    fixture_filename = file_relative_path(
        __file__,
        "./fixtures/ValidationResultsPageRenderer_render_with_run_info_at_end.json",
    )
    with open(fixture_filename, "r") as infile:
        rendered_validation_results = json.load(infile)
        return rendered_validation_results


@pytest.fixture()
def ValidationResultsPageRenderer_render_with_run_info_at_start():
    """
    Rendered validation results with run info at the start
    Returns:
        json string of rendered validation results
    """
    fixture_filename = file_relative_path(
        __file__,
        "./fixtures/ValidationResultsPageRenderer_render_with_run_info_at_start.json",
    )
    with open(fixture_filename, "r") as infile:
        rendered_validation_results = json.load(infile)
        return rendered_validation_results


def test_snapshot_ValidationResultsPageRenderer_render_with_run_info_at_end(
    titanic_profiled_evrs_1, ValidationResultsPageRenderer_render_with_run_info_at_end,
):
    validation_results_page_renderer = ValidationResultsPageRenderer(
        run_info_at_end=True
    )
    rendered_validation_results = validation_results_page_renderer.render(
        titanic_profiled_evrs_1
    ).to_json_dict()
    print(rendered_validation_results)
    # with open(file_relative_path(__file__, "./fixtures/ValidationResultsPageRenderer_render_with_run_info_at_end.json"), "w") as f:
    #     json.dump(rendered_validation_results, f, indent=2)

    assert (
        rendered_validation_results
        == ValidationResultsPageRenderer_render_with_run_info_at_end
    )


def test_snapshot_ValidationResultsPageRenderer_render_with_run_info_at_start(
    titanic_profiled_evrs_1,
    ValidationResultsPageRenderer_render_with_run_info_at_start,
):
    validation_results_page_renderer = ValidationResultsPageRenderer(
        run_info_at_end=False
    )
    rendered_validation_results = validation_results_page_renderer.render(
        titanic_profiled_evrs_1
    ).to_json_dict()
    print(rendered_validation_results)
    # with open(file_relative_path(__file__, "./fixtures/ValidationResultsPageRenderer_render_with_run_info_at_start.json"), "w") as f:
    #     json.dump(rendered_validation_results, f, indent=2)

    assert (
        rendered_validation_results
        == ValidationResultsPageRenderer_render_with_run_info_at_start
    )



import datetime

import boto3
import pytest
from freezegun import freeze_time
from marshmallow import ValidationError
from moto import mock_s3

from great_expectations.data_context.store import HtmlSiteStore
from great_expectations.data_context.types.resource_identifiers import (
    ExpectationSuiteIdentifier,
    SiteSectionIdentifier,
    ValidationResultIdentifier,
    validationResultIdentifierSchema,
)
from great_expectations.util import gen_directory_tree_str


@freeze_time("09/26/2019 13:42:41")
def test_HtmlSiteStore_filesystem_backend(tmp_path_factory):
    path = str(
        tmp_path_factory.mktemp(
            "test_HtmlSiteStore_with_TupleFileSystemStoreBackend__dir"
        )
    )

    my_store = HtmlSiteStore(
        store_backend={
            "class_name": "TupleFilesystemStoreBackend",
            "base_directory": "my_store",
        },
        runtime_environment={"root_directory": path},
    )

    with pytest.raises(TypeError):
        my_store.get("not_a_ValidationResultIdentifier")

    with pytest.raises(ValidationError):
        my_store.get(validationResultIdentifierSchema.load({}))

    ns_1 = SiteSectionIdentifier(
        site_section_name="validations",
        resource_identifier=ValidationResultIdentifier.from_tuple(
            (
                "a",
                "b",
                "c",
                "quarantine",
                datetime.datetime.now(datetime.timezone.utc),
                "prod-100",
            )
        ),
    )
    my_store.set(ns_1, "aaa")
    # assert my_store.get(ns_1) == "aaa"

    ns_2 = SiteSectionIdentifier(
        site_section_name="validations",
        resource_identifier=ValidationResultIdentifier.from_tuple(
            (
                "a",
                "b",
                "c",
                "quarantine",
                datetime.datetime.now(datetime.timezone.utc),
                "prod-20",
            )
        ),
    )
    my_store.set(ns_2, "bbb")
    # assert my_store.get(ns_2) == {"B": "bbb"}

    print(my_store.list_keys())
    # WARNING: OBSERVE THAT SITE_SECTION_NAME IS LOST IN THE CALL TO LIST_KEYS
    assert set(my_store.list_keys()) == {
        ns_1.resource_identifier,
        ns_2.resource_identifier,
    }

    print(gen_directory_tree_str(path))
    assert (
        gen_directory_tree_str(path)
        == """\
test_HtmlSiteStore_with_TupleFileSystemStoreBackend__dir0/
    my_store/
        validations/
            a/
                b/
                    c/
                        quarantine/
                            20190926T134241.000000Z/
                                prod-100.html
                                prod-20.html
"""
    )


@freeze_time("09/26/2019 13:42:41")
@mock_s3
def test_HtmlSiteStore_S3_backend():
    bucket = "test_validation_store_bucket"
    prefix = "test/prefix"

    # create a bucket in Moto's mock AWS environment
    conn = boto3.resource("s3", region_name="us-east-1")
    conn.create_bucket(Bucket=bucket)

    my_store = HtmlSiteStore(
        store_backend={
            "class_name": "TupleS3StoreBackend",
            "bucket": bucket,
            "prefix": prefix,
        }
    )

    with pytest.raises(TypeError):
        my_store.get("not_a_ValidationResultIdentifier")

    ns_1 = SiteSectionIdentifier(
        site_section_name="validations",
        resource_identifier=ValidationResultIdentifier(
            expectation_suite_identifier=ExpectationSuiteIdentifier(
                expectation_suite_name="asset.quarantine",
            ),
            run_id="20191007T151224.1234Z_prod_100",
            batch_identifier="1234",
        ),
    )
    my_store.set(ns_1, "aaa")

    ns_2 = SiteSectionIdentifier(
        site_section_name="expectations",
        resource_identifier=ExpectationSuiteIdentifier(
            expectation_suite_name="asset.quarantine",
        ),
    )
    my_store.set(ns_2, "bbb")

    assert set(my_store.list_keys()) == {
        ns_1.resource_identifier,
        ns_2.resource_identifier,
    }

    # This is a special un-store-like method exposed by the HtmlSiteStore
    my_store.write_index_page("index_html_string_content")

    # Verify that internals are working as expected, including the default filepath
    # paths below should include the batch_parameters
    assert set(
        [
            s3_object_info["Key"]
            for s3_object_info in boto3.client("s3").list_objects_v2(
                Bucket=bucket, Prefix=prefix
            )["Contents"]
        ]
    ) == {
        "test/prefix/index.html",
        "test/prefix/expectations/asset/quarantine.html",
        "test/prefix/validations/asset/quarantine/20191007T151224.1234Z_prod_100/20190926T134241.000000Z/1234.html",
    }

    index_content = (
        boto3.client("s3")
        .get_object(Bucket=bucket, Key="test/prefix/index.html")["Body"]
        .read()
        .decode("utf-8")
    )
    assert index_content == "index_html_string_content"



# -*- coding: utf-8 -*-
import pytest

from great_expectations.core import RunIdentifier
from great_expectations.data_context.types.resource_identifiers import (
    ExpectationSuiteIdentifier,
    ValidationResultIdentifier,
)
from great_expectations.render.util import (
    num_to_str,
    resource_key_passes_run_name_filter,
)


def test_num_to_str():
    f = 0.99999999999999
    # We can round
    assert num_to_str(f, precision=4) == "≈1"
    # Specifying extra precision should not cause a problem
    assert num_to_str(f, precision=20) == "0.99999999999999"

    f = 1234567890.123456  # Our float can only hold 17 significant digits
    assert num_to_str(f, precision=4) == "≈1.235e+9"
    assert num_to_str(f, precision=20) == "1234567890.123456"
    assert num_to_str(f, use_locale=True, precision=40) == "1,234,567,890.123456"

    f = 1.123456789012345  # 17 sig digits mostly after decimal
    assert num_to_str(f, precision=5) == "≈1.1235"
    assert num_to_str(f, precision=20) == "1.123456789012345"

    f = 0.1  # A classic difficulty for floating point arithmetic
    assert num_to_str(f) == "0.1"
    assert num_to_str(f, precision=20) == "0.1"
    assert num_to_str(f, no_scientific=True) == "0.1"

    f = 1.23456789012345e-10  # significant digits can come late
    assert num_to_str(f, precision=20) == "1.23456789012345e-10"
    assert num_to_str(f, precision=5) == "≈1.2346e-10"
    assert (
        num_to_str(f, precision=20, no_scientific=True) == "0.000000000123456789012345"
    )
    assert num_to_str(f, precision=5, no_scientific=True) == "≈0.00000000012346"

    f = 100.0  # floats should have trailing digits and numbers stripped
    assert num_to_str(f, precision=10, no_scientific=True) == "100"
    assert num_to_str(f, precision=10) == "100"
    assert num_to_str(f, precision=10, use_locale=True) == "100"

    f = 100  # integers should never be stripped!
    assert num_to_str(f, precision=10, no_scientific=True) == "100"
    assert num_to_str(f, precision=10) == "100"
    assert num_to_str(f, precision=10, use_locale=True) == "100"

    f = 1000  # If we have a number longer than our precision, we should still be able to correctly format
    assert num_to_str(f, precision=4) == "1000"
    assert num_to_str(f) == "1000"


def test_resource_key_passes_run_name_filter():
    resource_key = ValidationResultIdentifier(
        expectation_suite_identifier=ExpectationSuiteIdentifier("test_suite"),
        run_id=RunIdentifier(run_name="foofooprofilingfoo"),
        batch_identifier="f14c3d2f6e8028c2db0c25edabdb0d61",
    )

    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"equals": "profiling"}
        )
        is False
    )
    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"equals": "foofooprofilingfoo"}
        )
        is True
    )

    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"not_equals": "profiling"}
        )
        is True
    )
    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"not_equals": "foofooprofilingfoo"}
        )
        is False
    )

    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"includes": "profiling"}
        )
        is True
    )
    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"includes": "foobar"}
        )
        is False
    )

    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"not_includes": "foobar"}
        )
        is True
    )
    assert (
        resource_key_passes_run_name_filter(
            resource_key, run_name_filter={"not_includes": "profiling"}
        )
        is False
    )

    assert (
        resource_key_passes_run_name_filter(
            resource_key,
            run_name_filter={"matches_regex": "(foo){2}profiling(" "foo)+"},
        )
        is True
    )
    assert (
        resource_key_passes_run_name_filter(
            resource_key,
            run_name_filter={"matches_regex": "(foo){3}profiling(" "foo)+"},
        )
        is False
    )
    with pytest.warns(DeprecationWarning):
        assert (
            resource_key_passes_run_name_filter(
                resource_key, run_name_filter={"eq": "profiling"}
            )
            is False
        )
        assert (
            resource_key_passes_run_name_filter(
                resource_key, run_name_filter={"eq": "foofooprofilingfoo"}
            )
            is True
        )
    with pytest.warns(DeprecationWarning):
        assert (
            resource_key_passes_run_name_filter(
                resource_key, run_name_filter={"ne": "profiling"}
            )
            is True
        )
        assert (
            resource_key_passes_run_name_filter(
                resource_key, run_name_filter={"ne": "foofooprofilingfoo"}
            )
            is False
        )



import pytest

from great_expectations.data_context import BaseDataContext
from great_expectations.data_context.types.base import DataContextConfig


@pytest.fixture(scope="module")
def basic_data_context_config_for_validation_operator():
    return DataContextConfig(
        config_version=2,
        plugins_directory=None,
        evaluation_parameter_store_name="evaluation_parameter_store",
        expectations_store_name="expectations_store",
        datasources={},
        stores={
            "expectations_store": {"class_name": "ExpectationsStore"},
            "evaluation_parameter_store": {"class_name": "EvaluationParameterStore"},
            "validation_result_store": {"class_name": "ValidationsStore"},
            "metrics_store": {"class_name": "MetricStore"},
        },
        validations_store_name="validation_result_store",
        data_docs_sites={},
        validation_operators={
            "store_val_res_and_extract_eval_params": {
                "class_name": "ActionListValidationOperator",
                "action_list": [
                    {
                        "name": "store_validation_result",
                        "action": {
                            "class_name": "StoreValidationResultAction",
                            "target_store_name": "validation_result_store",
                        },
                    },
                    {
                        "name": "extract_and_store_eval_parameters",
                        "action": {
                            "class_name": "StoreEvaluationParametersAction",
                            "target_store_name": "evaluation_parameter_store",
                        },
                    },
                ],
            },
            "errors_and_warnings_validation_operator": {
                "class_name": "WarningAndFailureExpectationSuitesValidationOperator",
                "action_list": [
                    {
                        "name": "store_validation_result",
                        "action": {
                            "class_name": "StoreValidationResultAction",
                            "target_store_name": "validation_result_store",
                        },
                    },
                    {
                        "name": "extract_and_store_eval_parameters",
                        "action": {
                            "class_name": "StoreEvaluationParametersAction",
                            "target_store_name": "evaluation_parameter_store",
                        },
                    },
                ],
            },
        },
    )


@pytest.fixture(scope="module")
def basic_in_memory_data_context_for_validation_operator(
    basic_data_context_config_for_validation_operator,
):
    return BaseDataContext(basic_data_context_config_for_validation_operator)



import copy
import json
import os
from collections import OrderedDict

import numpy as np
import pandas as pd
import pytest

from great_expectations.data_asset.util import recursively_convert_to_json_serializable

from ..test_utils import (
    candidate_getter_is_on_temporary_notimplemented_list,
    get_dataset,
)

dir_path = os.path.dirname(os.path.realpath(__file__))
test_config_path = os.path.join(dir_path, "test_dataset_implementations.json")
test_config = json.load(open(test_config_path), object_pairs_hook=OrderedDict)
test_datasets = test_config["test_datasets"]


def generate_ids(test):
    return ":".join([test["dataset"], test["func"]])


@pytest.mark.parametrize(
    "test", test_config["tests"], ids=[generate_ids(t) for t in test_config["tests"]]
)
def test_implementations(test_backend, test):
    should_skip = candidate_getter_is_on_temporary_notimplemented_list(
        test_backend, test["func"]
    ) or test_backend in test.get("suppress_test_for", [])
    if should_skip:
        pytest.skip()

    data = test_datasets[test["dataset"]]["data"]
    schema = test_datasets[test["dataset"]]["schemas"].get(test_backend)
    dataset = get_dataset(test_backend, data, schemas=schema)
    func = getattr(dataset, test["func"])
    run_kwargs = copy.deepcopy(test.get("kwargs", {}))
    for arg in run_kwargs.keys():
        if isinstance(run_kwargs[arg], list):
            run_kwargs[arg] = tuple(run_kwargs[arg])
    result = func(**run_kwargs)

    # NOTE: we cannot serialize pd.Series to json directly,
    # so we're going to test our preferred serialization.
    # THIS TEST DOES NOT REPRESENT THE EXPECTED RETURN VALUE
    # OF THE TESTED FUNCTION; THIS IS A JOINT TEST OF THE
    # JSON SERIALIZATION AND THE TEST.
    # See test_get_column_value_counts for a series-specific test
    if test["func"] == "get_column_value_counts":
        result = recursively_convert_to_json_serializable(result)

    if "tolerance" in test:
        assert np.allclose(test["expected"], result, test["tolerance"])
    elif isinstance(test["expected"], list):
        if len(test["expected"]) > 0 and isinstance(test["expected"][0], dict):
            for item in test["expected"]:
                assert item in result
        else:
            assert test["expected"] == result
    else:
        assert test["expected"] == result


def test_get_column_value_counts(test_backend):
    schemas = {
        "SparkDFDataset": {
            "x": "FloatType",
            "y": "IntegerType",
            "z": "IntegerType",
            "n": "IntegerType",
            "b": "BooleanType",
        },
        "mysql": {
            "x": "DOUBLE",
            "y": "INTEGER",
            "z": "INTEGER",
            "n": "INTEGER",
            "b": "TINYINT",
        },
        "mssql": {
            "x": "FLOAT",
            "y": "INTEGER",
            "z": "INTEGER",
            "n": "INTEGER",
            "b": "BIT",
        },
    }
    data = {
        "x": [2.0, 5.0],
        "y": [5, 5],
        "z": [0, 10],
        "n": [0, None],
        "b": [True, False],
    }
    dataset = get_dataset(test_backend, data, schemas=schemas)

    res = dataset.get_column_value_counts("x")
    expected = pd.Series(data["x"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"

    assert res.equals(expected)

    res = dataset.get_column_value_counts("y")
    expected = pd.Series(data["y"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    res = dataset.get_column_value_counts("z")
    expected = pd.Series(data["z"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    res = dataset.get_column_value_counts("n")
    expected = pd.Series(data["n"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    res = dataset.get_column_value_counts("b")
    expected = pd.Series(data["b"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    data = {
        "a": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "b": ["a", "b", "b", "c", "c", "c", "d", "d", "d", "d"],
        "c": ["a", "b", "b", "c", "c", "c", "d", None, None, None],
        "d": ["a", "b", "c", "d", "e", "f", "g", None, None, None],
    }
    schemas = {
        "SparkDFDataset": {
            "a": "IntegerType",
            "b": "StringType",
            "c": "StringType",
            "d": "StringType",
        }
    }
    dataset = get_dataset(test_backend, data, schemas=schemas)

    res = dataset.get_column_value_counts("a")
    expected = pd.Series(data["a"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    res = dataset.get_column_value_counts("b")
    expected = pd.Series(data["b"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    res = dataset.get_column_value_counts("c")
    expected = pd.Series(data["c"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)

    res = dataset.get_column_value_counts("d")
    expected = pd.Series(data["d"]).value_counts()
    expected.sort_index(inplace=True)
    expected.index.name = "value"
    expected.name = "count"
    assert res.equals(expected)
